/*
 * ColorRange
 *
 * Author: M. Heidi McClure
 * Date Started: 10/24/96
 * Date Last Revised: 4/12/97
 *
 */

import java.awt.*;
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890

/**
 * This class handles the color ranges
 * used to represent a user's power in the UTMOST2 model.  These colors
 * are used for the color of the users and are used in the power legend.
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 * @see Legend
 * @see User
 */

public class ColorRange {
    private double lo;
    private double hi;
    private Color clr;

    /**
     * Constructor which will set the low and high power limit of Color c.
     *
     * @param l is the lower limit of the power range.
     * @param h is the high limit of the power range.
     * @param c is the color to which this range is associated
     */
    public ColorRange(double l, double h, Color c) {
        lo = l;
        hi = h;
        clr = c;
    }

    public void setLo(float f) {
        lo = f;
    }

    public double getLo() {
        return lo;
    }

    public void setHi(float f) {
        hi = f;
    }

    public double getHi() {
        return hi;
    }

    public void setColor(Color i) {
        clr = i;
    }

    public Color getColor() {
        return clr;
    }
}
