/*
 * Console Dialog class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/12/97
 * Date Last Revised: 4/12/97
 */
/*
 * Console is a class which allows information to be printed to
 * the a console-like window.  Anywhere a System.out.println is
 * used in UTMOST2, a Console printout is performed if the Console
 * exists.
 */

import java.awt.*;

/**
 * This class generates a console window when
 * selected from the UTMOST2 pull down menus.  The console will display
 * any message which is sent to the UTMOST2 asNeededPrint() or
 * asNeededPrintln() methods.
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 * @see UTMOST2#asNeededPrint()
 * @see UTMOST2#asNeededPrintln()
 */
public class Console extends Dialog {

    public TextArea out = new TextArea(24, 80);

    public Console(Frame parent, String title) {

        super(parent, title, false); // false makes a modeless dialog
                                    // so use of parent is NOT blocked

        out.setEditable(false);

        setLayout(new FlowLayout());

        add(out);

//        addNotify();

        // resize(insets().left + insets().right + 352, insets().top + insets().bottom + 254);
        resize(insets().left + insets().right + 750,
                insets().top + insets().bottom + 450);

        // show();  // cannot show now.  If you do, then can never get
                    // this window to go behind the UTMOST2 window.  I
                    // don't know why.  Also screws up displaying of
                    // utmost menus!!!  could be an x86 or win95 bug.
        setResizable(false);
    }

    public synchronized void show() {
        move(50, 50);
        super.show();
    }
    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event event) {

        if (event.id == Event.WINDOW_DESTROY) {
            hide();         // hide the Frame
            dispose();      // tell windowing system to free resources
                            // works for win95, but not UNIX
            // System.exit(0); // don't exit since console isn't standalone.
            return true;
        }
        return super.handleEvent(event);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event event, Object arg) {
        if (event.target instanceof MenuItem) {
            String label = (String) arg;
        }
        return super.action(event, arg);
    }

    public synchronized void println(String s) {
        out.appendText(s);
        out.appendText("\n");
    }

    public synchronized void print(String s) {
        out.appendText(s);
    }

/*
 * This main is strictly test code!  (for Console Class)
 */
    public static void main(String args[]) {
        Console myConsole = new
            Console(new Frame(), "Console Window");
        for (int i = 0; i < 1000; i++) {
            myConsole.println("" + i + " - more text");
        }
    }

}

