/*
 * Legend window
 *
 * Author: M. Heidi McClure
 * Date Started: 10/24/96
 * Date Last Revised: 4/12/97
 */

import java.awt.*;
// import gjt.Box;
/**
 * This class will show the panel of the Power legend on the MapPanel and
 * next to the MapArea.
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 * @see ColorRange
 * @see MapPanel
 * @see MapArea
 */
public class Legend extends Panel {

    /**
     * colors is the array used tracking all of the ranges of colors.
     * <p>
     * It is set by the constructor in this class
     * <p>
     * It is used by the drawUsers() method in mapArea class for showing
     * a user's power visually by its color.
     * @see MapArea#drawUsers
     * @see User
     */
    public ColorRange [] colors;
    private Panel key = new Panel();
    private Panel center = new Panel();
    private Panel west = new Panel();
    private Label [] colorLabel = new Label [7];
    private ColorCanvas [] dots = new ColorCanvas [7];

    private UTMOST2 the;

    private int dotSize;
    private double incr;

    private Font myFont = new Font("Helvetica", Font.PLAIN, 10);

    /**
     * Constructor which will display the Power Legend.
     * <p>
     * Currently the range is from 0 to 3 watts in .5 watt increments.
     *
     * @param parent is the caller of this class.
     */
    public Legend(UTMOST2 p) {
//    super("Legend"); // this was used when Legend extended Dialog and
                        // displayed a stand-alone window.
        int i;
	int inc;
        the = p;
        dotSize = the.DOT_SIZE;
	incr = the.POWER_MAX / 6;

        setLayout(new BorderLayout());
	
        colors = new ColorRange [7];
        colors[0] = new ColorRange(0, 0, Color.black);
        colors[1] = new ColorRange(0, incr, Color.gray);
        colors[2] = new ColorRange(incr, incr * 2, Color.pink.darker());
        colors[3] = new ColorRange(incr * 2, incr * 3, Color.pink);
        colors[4] = new ColorRange(incr * 3, incr * 4, Color.orange);
        colors[5] = new ColorRange(incr * 4, incr * 5, Color.red);
        colors[6] = new ColorRange(incr * 5, incr * 6, Color.magenta);

        west.setLayout(new GridLayout(7,1,0,0));
        for (i = 0; i < 7; i++) {
            dots[i] = new ColorCanvas(colors[i].getColor(), dotSize);
            dots[i].resize(10,24);
            west.add(dots[i]);
        }

        center.setLayout(new GridLayout(7,1,0,0));
        colorLabel[0] = new Label("x = 0");
        center.add(colorLabel[0]);

        for (i = 1; i < 7; i++) {
            colorLabel[i] = new Label(colors[i].getLo() + " < x <= " +
                colors[i].getHi());
            center.add(colorLabel[i]);
        }

        key.add("West", west);
        key.add("East", center);

        EtchBox keyBox = new EtchBox(key, "Power (Watts)");
        add("Center", keyBox);

        resize( insets().left + insets().right + 250,
                insets().top + insets().bottom + 175);
//        setResizable(false);

//        validate(); not sure why this is needed anyways
//        show();
    }

    public void updateLegend() {
        int i;
	int inc;
        dotSize = the.DOT_SIZE;
	incr = the.POWER_MAX / 6;

	hide();
	invalidate();
	removeAll();
	key.removeAll();
	west.removeAll();
	center.removeAll();
	
        colors = new ColorRange [7];
        colors[0] = new ColorRange(0, 0, Color.black);
        colors[1] = new ColorRange(0, incr, Color.gray);
        colors[2] = new ColorRange(incr, incr * 2, Color.pink.darker());
        colors[3] = new ColorRange(incr * 2, incr * 3, Color.pink);
        colors[4] = new ColorRange(incr * 3, incr * 4, Color.orange);
        colors[5] = new ColorRange(incr * 4, incr * 5, Color.red);
        colors[6] = new ColorRange(incr * 5, incr * 6, Color.magenta);

        for (i = 0; i < 7; i++) {
            dots[i] = new ColorCanvas(colors[i].getColor(), dotSize);
            dots[i].resize(10,24);
            west.add(dots[i]);
        }

        colorLabel[0] = new Label("x = 0");
        center.add(colorLabel[0]);

        for (i = 1; i < 7; i++) {
            colorLabel[i] = new Label(
		(new Format("%4.2f").form(colors[i].getLo())) + " < x <= " +
                (new Format("%4.2f").form(colors[i].getHi())));
            center.add(colorLabel[i]);
//	    System.out.println("colorLabel[i].getText: " +
//				colorLabel[i].getText());
        }

        key.add("West", west);
        key.add("East", center);

        EtchBox keyBox = new EtchBox(key, "Power (Watts)");
        add("Center", keyBox);

//        resize( insets().left + insets().right + 250,
//                insets().top + insets().bottom + 175);
	show();
	getParent().validate();
//        setResizable(false);

    }

    public synchronized void show() {
        move(500, 175);
        super.show();
    }

/*
    public void paint(Graphics g) {
        // draw Legend Dots...
        g.setFont(myFont);
//        for (int i = 1; i < 7; i++) {
//            dots[i].paint(g);
//        }

        g.setColor(colors[0].getColor());
        g.fillOval(20, 30, dotSize, dotSize);

        for (int i = 1; i < 7; i++) {
            g.setColor(colors[i].getColor());
            g.fillOval(20, (int)(i*19.9 + 30), dotSize, dotSize);
        }

    }
*/
    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event event) {
        return super.handleEvent(event);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event event, Object arg) {
        return super.action(event, arg);
    }
}
/**
 * This class is used by the Legend class (only) in an attempt to align
 * the dots to the text better.  It will draw a dot of a specified color
 * and size.
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see Legend
 */
class ColorCanvas extends Canvas {
    int dotSize;
    Color color;

    public void paint (Graphics g) {
        g.setColor(color);
        g.fillOval(0, 8, dotSize, dotSize);
    }
    /**
     * Constructor which will set the Color and the dotSize.
     * <p>
     * Currently the range is from 0 to 3 watts in .5 watt increments.
     *
     * @param c is the Color of the dot.
     * @param d is the dot height and width
     */
    public ColorCanvas(Color c, int d) {
        super();
        setDotSize(d);
        setColor(c);
    }
    public void setDotSize(int d) {
        dotSize = d;
    }
    public void setColor(Color c) {
        color = c;
    }
}
