/*
 * PropertySettings class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/12/97
 * Date Last Revised: 4/12/97
 */

/**
 * PropertySettings is a class which allows variables used in UTMOST2 model
 * to be changed.  (Cannot use the word Properties since this is a java class
 * in the JDK... not worth making sure the CLASSPATH is correct).
 * <p>
 * This class does not function completely.  It merely provides the
 * hooks to perform some parameterization of some of the variables used
 * in the model.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see VariablesPanel
 * @see FilesPanel
 * @see UsersPanel
 * @see SectorsPanel
 */

import java.awt.*;

public class LoadDialog extends Dialog {

    private Button OpenButton = new Button("Open");
	private Button CancelButton = new Button("Cancel");

	private Label dateLabel = new Label("Open Date:");
	private TextField dateText =  new TextField("");

	private String date;

	private UTMOST2 the;

    /**
     * Constructor which will create a PropertySettings object.
     *
     * @param t is the Frame object to which this dialog belongs.
     */
    public LoadDialog(Frame t) {
        super(t, "Open .. ", true);
		
        the = (UTMOST2) t;
		setLayout(null);
        //addNotify();
        resize( insets().left + insets().right + 350,
                insets().top + insets().bottom + 180);
        
        
        add(dateLabel);
        dateLabel.reshape( insets().left + 12,
                        insets().top + 65,80,20);
		add(dateText);
        dateText.reshape( insets().left + 100,
                        insets().top + 65,100,30);

        add(OpenButton);
        OpenButton.reshape(insets().left + 90,insets().top + 120,70,26);
		add(CancelButton);
        CancelButton.reshape(insets().left + 175,insets().top + 120,70,26);

    }


	public String getDate(){
		return date;
	}

    public synchronized void show() {
        move(50, 50);
        super.show();
    }

    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event evt) {
        if (evt.id == Event.WINDOW_DESTROY) {
            hide();
            // dispose();
            return true;
        }
        return super.handleEvent(evt);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        
        
        if (evt.target == OpenButton){
			date = new String((dateText.getText()).trim());
			if(date!=null)
				the.selectedUser.readModelDataFile(date);
			hide();
			return super.action(evt, arg);
		}
		if (evt.target == CancelButton){
			hide();
			return super.action(evt, arg);
		}
		
		return super.action(evt, arg);
    }

	public static void main(String [] args) {
		LoadDialog myProp = new LoadDialog(new Frame());
        myProp.show();
    }

}
