/*
 * MapArea Canvas
 *
 * Author: M. Heidi McClure
 * Date Started: 9/1/96
 * Date Last Revised: 4/12/97
 */

import java.awt.*;
/**
 * This class draws the hexagon patterns, the users and other information
 * about the current UTMOST2 model.  It is a component which is part of
 * the MapPanel class.
 * <p>
 *
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see MapPanel
 * @see User
 * @see UTMOST2
 * @see UserStats
 * @see ModelSetup
 */
public class MapArea extends Canvas {
    /*
     * (X[i]*radius) + centerX = xCoord for hexagon placement
     *
     * Then adjust for zoom factor...
     */
// the arrays below take into account the coordinate system needed for hex's

    private double X[] = {-0.866, -0.866, 0, 0.866, 0.866, 0, -0.866};
    private double Y[] = {-0.500, 0.5, 1.0, 0.5, -0.500, -1.000, -0.500};

    private int points = X.length;

    private double radius;
    private double scaler = 13.2;  // will be entered zoom divided by radius
    private int inset = 10; // how far in and down to start model drawing
    private int numUsers = 25;

    private int dotSize;
    private int halfDot;

    private UTMOST2 the;

    private int curUserIndex;
    private boolean shouldShowSectors;
    private boolean shouldShowAllAntennae;
    private boolean shouldShowChangedAntennae;
    private boolean shouldShowChangedPower;

	Toolkit toolkit;
	Image image;

	private int Xoffset=0; //offset from moving cells
	private int Yoffset=0;
	private int oldXoffset=0; 
	private int oldYoffset=0;
	private int MapXoffset=0; //offset from scrolling image
	private int MapYoffset=0;
	private int repaintCount=0;
	
	private int mouseDownX;
	private int mouseDownY;
	private boolean moveMap = false;

// Constructors
    public MapArea() {
        // use default FlowLayout
    }
    /**
     * Constructor which will create a MapArea object.
     *
     * @param u is the UTMOST2 object which called this constructor
     */
    public MapArea(UTMOST2 u) {
        the = u;
        radius = the.RADIUS;
        dotSize = the.DOT_SIZE;
        halfDot = dotSize / 2;
        shouldShowSectors = false;
        shouldShowAllAntennae = false;
        shouldShowChangedAntennae = false;
        shouldShowChangedPower = false;
		//if (the.drawImage)
			setupImage();
    }

    public void setUTMOST2(UTMOST2 u) {
        the = u;
    }

	public void changeMoveMap(){
		if(moveMap)
			moveMap = false;
		else
			moveMap = true;
	}

	public void setupImage(){
		toolkit = the.getToolkit();
		image = toolkit.getImage(the.ImageName);
	}

	public void setOffset(int x,int y){
		Xoffset = oldXoffset + x;
		Yoffset = oldYoffset + y;
		the.model.setOffsetText(Xoffset,Yoffset);
	}

	public void updateOffset(int x,int y){
		Xoffset = x;
		Yoffset = y;
		the.model.setOffsetText(Xoffset,Yoffset);
	}

	public void setMapOffsets(int x,int y){
		MapXoffset = x;
		MapYoffset = y;
		repaint();
	}

	public int getMapXoffset(){
		return MapXoffset;
	}

	public int getMapYoffset(){
		return MapYoffset;
	}


    public void setShouldShowSectors(boolean s) {
        shouldShowSectors = s;
    }
    public boolean getShouldShowSectors() {
        return shouldShowSectors;
    }

    public void setShouldShowAllAntennae(boolean s) {
        shouldShowAllAntennae = s;
    }
    public boolean getShouldShowAllAntennae() {
        return shouldShowAllAntennae;
    }

    public void setShouldShowChangedAntennae(boolean s) {
        shouldShowChangedAntennae = s;
    }
    public boolean getShouldShowChangedAntennae() {
        return shouldShowChangedAntennae;
    }

    public void setShouldShowChangedPower(boolean s) {
        shouldShowChangedPower = s;
    }
    public boolean getShouldShowChangedPower() {
        return shouldShowChangedPower;
    }

    public void setNumUsers(int n) {
        numUsers = n;
    }
    public int getNumUsers() {
        return numUsers;
    }
    /**
     * When the mouse is pressed down, if it is inside a user dot,
     * information about this user needs to be displayed in the
     * UserStats area of the UTMOST2 model.  Also, the current serving
     * antenna sector and all visible antennae sectors will be
     * displayed by shading those sectors.
     *
     */
    public boolean mouseDown(Event event, int x, int y) {
        
		if(moveMap){
			mouseDownX = x;
			mouseDownY = y;
			oldXoffset=Xoffset;
			oldXoffset=Xoffset;
		}
		else {
			int i;
			int ux;
			int uy;
			Rectangle tempRect = new Rectangle(0,0,dotSize,dotSize);

			the.asNeededPrintln("MouseDown x = " + x + " y = " + y);

	        x = x - inset+MapXoffset-Xoffset;
		    y = y - inset+MapYoffset-Yoffset;
			for (i = 0; i < numUsers; i++) {
				ux = (int)((scaler * the.users[i].getCurXLocation()) - halfDot);
	            uy = (int)((scaler * the.users[i].getCurYLocation()) - halfDot);
		        tempRect.move(ux, uy);
			    if (tempRect.inside(x, y)) {
				    the.asNeededPrintln("  inside User number " + i + " " +
					    the.users[i]);
					the.selectedUser.updateFields(the.users[i]);
					curUserIndex = i;
					showVisibleSectors(the.users[i], this.getGraphics());
					break; // even though there may be more than one user under
                        // the mouse, only display info for one - otherwise
                        // the visible sector display can get screwed up.
				}	
			}
		}
        return true;
		
    }

    /**
     * When the mouse is released, all visible sectors must be
     * undrawn (hidden).
     */

    public boolean mouseUp(Event event, int x, int y) {

		if(moveMap){
			setOffset(x-mouseDownX,y-mouseDownY);
			repaint();
		}
		else{
			// since showing is an XOR operations, calling show again will delete
			// the visible sectors
			if (curUserIndex != -1) {
				showVisibleSectors(the.users[curUserIndex], this.getGraphics());
			}
		curUserIndex = -1;
		}
        return true;
    }

	public boolean mouseDrag(Event  evt, int  x, int  y){
		if(moveMap){
			setOffset(x-mouseDownX,y-mouseDownY);
			/*if(x>400)
				the.mapPanel.setHScrollbarValue(10);
			if(y>400)
				the.mapPanel.setVScrollbarValue(10);*/
			repaintCount++;
			if(repaintCount  == 5){
				repaint();
				repaintCount = 0;
			}
		}
		return true;
	}


/*
 * futures:  would be nice to draw Scale and Hex only when the scale or pattern
 * has been changed.
 */
    /**
     * Everytime the MapArea needs to be redrawn, must draw all
     * desired things.
     * <p>
     * Depending on whether they are selected to
     * be shown or not, paint will show sector names, users current
     * antenna assignment, users who have just changed antenna
     * assignment, and users who have changed power assignment.
     * <p>
     * The scale, the sectors and the users will always be shown.
     */
    public void paint(Graphics g) {
        dotSize = the.DOT_SIZE;
        halfDot = dotSize / 2;
//        radius = the.RADIUS; // cannot do until sectors NOT read in
                                // from the file!!!

		// 
		if (the.drawImage){
			int x=the.imageX-MapXoffset;
			int y=the.imageY-MapYoffset;
			g.drawImage(image,x,y,this);
		}
		//
		if (shouldShowSectors) {
            drawSectorNames(g);
        }
        drawScale(g);
        drawHex(g);
        drawUsers(g);
        if (shouldShowAllAntennae) {
            drawAllAntennae(g);
        }
        if (shouldShowChangedAntennae) {
            drawChangedAntennae(g);
        }
        if (shouldShowChangedPower) {
            drawChangedPower(g);
        }

    }

    public void update(Graphics g) {
        super.update(g);  // this purges the old stuff off the display
                          // this was not working on SGI machines.

        paint(g);         // this draws the new location of users
    }

    /**
     * This method is called when the display needs to be updated, but
     * none of the model setup parameters have changed.
     */
    public void updateMap() {        // do not alter anything
        try {
            repaint();
        } catch(Exception e) {
            // need to print out something...
            /*
            MessageDialog d = MessageDialog.getMessageDialog(
                                Util.getFrame(this),
                                (DialogClient)this,
                                "Error",
                                "blah, blah, blah");
            d.pack();
            d.show();
            */

        }
    }

    /**
     * This method is called when the zoom factor has been changed.
     * <p>
     * Note that 66 is the default zoom when the model starts.
     *
     * @param zoom the zoom factor desired.
     * @see ModelSetup#action
     */
    public void updateMap(double zoom) {
    	// set cursor back to original cursor type
    	the.setCursor(Frame.WAIT_CURSOR);

        scaler = zoom/radius;
        try {
            repaint();
        } catch(Exception e) {
            // need to print out something...
            /*
            MessageDialog d = MessageDialog.getMessageDialog(
                                Util.getFrame(this),
                                (DialogClient)this,
                                "Error",
                                "blah, blah, blah");
            d.pack();
            d.show();
            */
        }
    	// set cursor back to original cursor type
    	the.setCursor(Frame.DEFAULT_CURSOR);
    }
    /**
     * This method is called when the type changes (Arrive, Leaving,
     * Random) or the number of users changes, or the cell pattern
     * changes, or if Go pressed and the model has not
     * been initialized for these parameters, yet.
     * <p>
     * Note that when this method is called, all users are reinitialized
     * based on the current model settings.
     * <p>
     * The possible models allowed in UTMOST2 are:
     * <p>
     * 7 Cell (circular pattern) <br>
     * Random Arriving <br>
     * Edge Arriving <br>
     * Random Leaving <br>
     * Center Leaving <br>
     * Random <br>
     * <p>
     * 4 Cell (linear pattern)
     * Random Arriving <br>
     * Arriving to Right <br>
     * Random Leaving <br>
     * Leaving to Left <br>
     * Random <br>
     *
     * <p>
     * @param model is the type (e.g. Arriving, Leaving, Random)
     *
     * @param pattern is the kind of cell pattern (e.g. 7 Cell or 4 Cell)
     * This may be changed to Circular or Linear in the future.
     *
     * @param behavior is the behavior of the users (e.g. Constant or
     * Regenerate).  Constant means users never change direction, speed
     * or other parameters even when they leave the current pattern.
     * Regenerate means that when users leave the bounds of the current
     * pattern, they are regenerated as a totally new user in another
     * area of the model, based on selected model parameters (i.e. they
     * are reinitialized)
     *
     * @param start is the initial starting position of a user - Random,
     * Center or Edge (Perimeter).
     *
     * @param u is the number of users currently in the model.
     *
     * @param zoom is the zoom factor of the current display.
     *
     * @see ModelSetup#action
     * @see UserStats#goButtonAction
     */
    public void updateMap(String model, String pattern, String behavior,
                          String start, int u, double zoom) {
    	// set cursor back to original cursor type
    	the.setCursor(Frame.WAIT_CURSOR);

        scaler = zoom/radius;
        numUsers = u;
        try {
            for (int i = 0; i < numUsers; i++) {
                // System.out.println("Calling setInitUser from updateMap");
                the.users[i].setInitUser(model, pattern, start,
                    the.model.getLineSeg());
            }
            the.selectedUser.setCurStep(0);
            the.selectedUser.setFirstGo(true);
            the.mapPanel.setDirText(" ");
            the.selectedUser.clearAllFields();
            repaint();
        } catch(Exception e) {
            // need to print out something...
            /*
            MessageDialog d = MessageDialog.getMessageDialog(
                                Util.getFrame(this),
                                (DialogClient)this,
                                "Error",
                                "blah, blah, blah");
            d.pack();
            d.show();
            */
        }
    	// set cursor back to original cursor type
    	the.setCursor(Frame.DEFAULT_CURSOR);
    }




    /**
     * This method is called to draw or redraw the scale of the model.
     * That is, how big 5km is in the model.
     * <p>
     * Currently 5km is hard coded here.  In the future this would be
     * generated based on a model parameter read in from one of the
     * PropertySettings panels.  (RADIUS will be the variable)
     * <p>
     * @see PropertySettings
     * @see VariablesPanel
     */
    public void drawScale(Graphics g) {
        double fiveK;
        int startX, startY;
        char [] string = new char [4];
        string[0] = '5';        // THIS IS WRONG.  Fix for UTMOST2 constants
        string[1] = 'k';
        string[2] = 'm';
        startX = inset;
        startY = 5;
        fiveK = (startX + (5 * scaler));
        g.setColor(Color.red);
        g.drawLine(startX, startY, (int)fiveK, startY);
        g.drawLine(startX, startY - 1, startX, startY + 1);
        g.drawLine((int)fiveK, startY - 1, (int)fiveK, startY + 1);
        g.drawChars(string, 0, 3, (int)(startX + scaler), startY + 15);
    }
    /**
     * This method is called to draw or redraw the hexagons and sectors
     * of the map.
     * <p>
     * This method uses two arrays, X[] and Y[] which contain exact
     * locations for placement of a unit hexagon.  This unit hexagon is
     * scaled to the proper size and offset by the center (x,y) of the
     * desired hexagon to be placed.
     * <p>
     */
    public void drawHex(Graphics g) {

        int i, j, r;
        double centerX, centerY;

        double cur_x[] = {0,0,0,0,0,0,0};
        double cur_y[] = {0,0,0,0,0,0,0};

        int int_x[] = {0,0,0,0,0,0,0};
        int int_y[] = {0,0,0,0,0,0,0};

        //setup initial x's and y's
        for (i = 0; i < points; i++) {
            cur_x[i] = X[i] * scaler * radius;
            cur_y[i] = Y[i] * scaler * radius;
        }
        //draw polygons
        for (i = 0; i < the.sector.length; i=i+3) {
            if (the.sector[i].getName() == -1) {
                break;
            }
            if (DebugLevel.debugging(DebugLevel.SECTOR_DEBUGGING)) {
                the.asNeededPrintln("draw Sect " + i);
                the.asNeededPrintln("   baseX = " + the.sector[i].getBaseX() +
                    " baseY = " + the.sector[i].getBaseY() +
                    " scaler = " + scaler);
            }
            centerX = (inset + the.sector[i].getBaseX() * scaler)-MapXoffset+Xoffset;
            centerY = (inset + the.sector[i].getBaseY() * scaler)-MapYoffset+Yoffset;

            //setup x's and y's based on centerX and Y
            for (j = 0; j < points; j++) {
                int_x[j] = (int)((centerX + cur_x[j]));
                int_y[j] = (int)((centerY + cur_y[j]));
            }

            g.setColor(Color.black);
            g.drawPolygon(int_x, int_y, points); //DRAW IT!!
            g.setColor(Color.green);
            g.drawLine((int)(centerX), (int)(centerY), int_x[5], int_y[5]);
            g.drawLine((int)(centerX), (int)(centerY), int_x[1], int_y[1]);
            g.drawLine((int)(centerX), (int)(centerY), int_x[3], int_y[3]);
        }
    }

    /**
     * This method is called to draw or redraw the users
     * of the current model.
     * <p>
     * The colors array setup in Legend is used to give each user a
     * color to indicate its current power.
     * <p>
     * Currently the size of the user dot is hard coded.  In the future
     * this would be
     * generated based on a model parameter read in from one of the
     * PropertySettings panels.  (DOT_SIZE will be the variable)
     * <p>
     * @see PropertySettings
     * @see VariablesPanel
     */
    public void drawUsers(Graphics g) {
        int i;
        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            the.asNeededPrintln("inside drawUsers, numUsers = " + numUsers);

            the.asNeededPrintln("draw Users[3] x = " +
            the.users[3].getCurXLocation() +
                "y = " + the.users[3].getCurYLocation());
        }
        for (i = 0; i < numUsers; i++) {
            if (Util.isEquiv(the.users[i].getCurPower(), 0)) {
                g.setColor(the.legend.colors[0].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[1].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[1].getHi()) {
                g.setColor(the.legend.colors[1].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[2].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[2].getHi()) {
                g.setColor(the.legend.colors[2].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[3].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[3].getHi()) {
                g.setColor(the.legend.colors[3].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[4].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[4].getHi()) {
                g.setColor(the.legend.colors[4].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[5].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[5].getHi()) {
                g.setColor(the.legend.colors[5].getColor());
            } else if (the.users[i].getCurPower() > the.legend.colors[6].getLo() &&
                       the.users[i].getCurPower() <= the.legend.colors[6].getHi()) {
                g.setColor(the.legend.colors[6].getColor());
            } else {
                g.setColor(Color.green);
            }
            int ovalX, ovalY;
            ovalX = (int)(inset + (the.users[i].getCurXLocation() * scaler) -
            halfDot-MapXoffset+Xoffset);
            ovalY = (int)(inset + (the.users[i].getCurYLocation() * scaler) -
            halfDot-MapYoffset+Yoffset);
/*
            if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                the.asNeededPrintln("Oval location: X: " + ovalX +
                                    " Y: " + ovalY);
            }
*/
            g.fillOval(ovalX, ovalY, dotSize, dotSize);
        }
    }

    /**
     * This method is called to draw or redraw the sector names in the
     * appropriate sector.
     * <p>
     * Whether or not to draw a sector is determined based on if the
     * shouldShowSectors variable has been set.  This variable is set
     * if the MapPanel button for "Show Sector Names" has been pressed.
     * <p>
     * @see MapPanel
     */
    public void drawSectorNames(Graphics g) {
        double x = 0;
        double y = 0;

        g.setColor(Color.black);
        for (int i = 0; i < the.sector.length; i++) {
            switch (the.sector[i].getCornerName()) {
                case 1 :
                    x = the.sector[i].getBaseX() -
                        (0.5 * the.sector[i].getRadius());
                    y = the.sector[i].getBaseY() -
                        (0.167 * the.sector[i].getRadius());
                    break;
                case 2 :
                    x = the.sector[i].getBaseX() +
                        (0.25 * the.sector[i].getRadius());
                    y = the.sector[i].getBaseY() -
                        (0.167 * the.sector[i].getRadius());
                    break;
                case 3 :
                    x = the.sector[i].getBaseX() -
                        (0.167 * the.sector[i].getRadius());
                    y = the.sector[i].getBaseY() +
                        (0.584 * the.sector[i].getRadius());
                    break;
                case -1 : // ignore when extra sectors are encountered.
                    break;
                default :
                    the.asNeededPrintln("Incorrect CornerName found: " +
                                        the.sector[i].getCornerName());
            }
            x = x * scaler + inset-MapXoffset+Xoffset;
            y = y * scaler + inset-MapYoffset+Yoffset;
            g.drawString(the.sector[i].getNameString(), (int)x, (int)y);
        }
    }

    /**
     * This method is called to draw or redraw the lines indicating
     * a user's currently antenna.
     * <p>
     * Whether or not to draw the antennae lines is determined based on
     * if the shouldShowAllAntennae variable has been set.  This variable
     * is set if the MapPanel button for "Show All Antennae" has been
     * pressed.
     * <p>
     * @see MapPanel
     */
    public void drawAllAntennae(Graphics g) {
        double x1 = 0;
        double y1 = 0;
        double x2 = 0;
        double y2 = 0;

        g.setColor(Color.darkGray);
        //System.err.println("noOfUsers="+the.map.getNumUsers());
        for (int i = 0; i < the.map.getNumUsers(); i++) {
            if (the.users[i].getCurSector() != -1) 
			{ // make sure inside model
                x1 = the.users[i].getCurXLocation() * scaler + inset-MapXoffset+Xoffset;
                y1 = the.users[i].getCurYLocation() * scaler + inset-MapYoffset+Yoffset;
                // System.err.println("currentSector="+the.users[i].getCurAntenna());
                if (the.users[i].getCurAntenna() > 0) {
                  x2 = the.sector[the.users[i].getCurAntenna()].getBaseX() *
                    scaler + inset-MapXoffset+Xoffset;
                  y2 = the.sector[the.users[i].getCurAntenna()].getBaseY() *
                    scaler + inset-MapYoffset+Yoffset;

                  g.drawLine((int)x1, (int)y1, (int)x2, (int)y2);
                }
            }
        }
    }

    /**
     * This method is called to draw or redraw the lines indicating
     * a user who has changed antenna assignments.  The current assigned
     * antenna will be shown with a red line, the previous antenna will
     * be shown with a green line.  If the user has moved to an antenna
     * with the same base station as the previous antenna, it will only
     * look like a red line is drawn.  The green is actually underneath
     * the red line.
     * <p>
     * Whether or not to draw the antennae lines is determined based on
     * if the shouldShowChangedAntennae variable has been set.  This
     * variable is set if the MapPanel button for "Show Changed
     * Antennae" has been pressed.
     * <p>
     * @see MapPanel
     */
    public void drawChangedAntennae(Graphics g) {
        double x1 = 0;
        double y1 = 0;
        double x2 = 0;
        double y2 = 0;

        for (int i = 0; i < the.map.getNumUsers(); i++) {
            if ((the.users[i].getCurSector() != -1) &&      // make sure inside
                    (the.users[i].getPrevSector() != -1)) 
			{ // the model
                if ((the.users[i].getPrevAntenna() !=
                        the.users[i].getCurAntenna()) &&
						(the.users[i].getCurAntenna() != -1) &&
						(the.users[i].getPrevAntenna() != -1)) 
				{
                    x1 = the.users[i].getCurXLocation() * scaler + inset-MapXoffset+Xoffset;
                    y1 = the.users[i].getCurYLocation() * scaler + inset-MapYoffset+Yoffset;

                    x2 = the.sector[the.users[i].getPrevAntenna()].getBaseX() *
                        scaler + inset-MapXoffset+Xoffset;
                    y2 = the.sector[the.users[i].getPrevAntenna()].getBaseY() *
                        scaler + inset-MapYoffset+Yoffset;

                    g.setColor(Color.green.darker().darker());
                    g.drawLine((int)x1, (int)y1, (int)x2, (int)y2);

                    x2 = the.sector[the.users[i].getCurAntenna()].getBaseX() *
                        scaler + inset-MapXoffset+Xoffset;
                    y2 = the.sector[the.users[i].getCurAntenna()].getBaseY() *
                        scaler + inset-MapYoffset+Yoffset;

                    g.setColor(Color.red);
                    g.drawLine((int)x1, (int)y1, (int)x2, (int)y2);
                }
            }
        }
    }

    /**
     * This method is called to draw or redraw the cicles around users
     * whose current power is different from previous power.
     * <p>
     * Whether or not to draw the circles is determined based on
     * if the shouldShowChangedPower variable has been set.  This
     * variable is set if the MapPanel button for "Show Changed
     * Power" has been pressed.
     * <p>
     * @see MapPanel
     */
    public void drawChangedPower(Graphics g) {
        double x = 0;
        double y = 0;

        double ovalSize = dotSize * 1.5;
        double ovalHalf = ovalSize / 2;

        g.setColor((Color.red).brighter());
        for (int i = 0; i < the.map.getNumUsers(); i++) {
            if (the.users[i].getPrevPower() != the.users[i].getCurPower()) {
                x = (the.users[i].getCurXLocation() * scaler + inset) -
                        ovalHalf-MapXoffset+Xoffset;
                y = (the.users[i].getCurYLocation() * scaler + inset) -
                        ovalHalf-MapYoffset+Yoffset;
                g.drawOval((int)x, (int)y, (int)ovalSize, (int)ovalSize);
            }
        }
    }

    /**
     * for a given (x, y) coordinate, calculate in which sector the point
     * is.  (x, y) is location read from user[i], so inset not a factor
     * in the calculations.
     * <p>
     * This method depends on the calcInside() method for figuring out if
     * a user is inside a particular sector boundary.  The built in
     * polygon method inside() was not sufficient because it is based
     * on integer values of (x,y).
     * <p>
     * Additionally, this method calls the onPerimeter() method to check
     * the special cases where a user is exactly on a point or line of
     * the cell pattern.
     * <p>
     * @see MapArea#calcInside
     * @see MapArea#onPerimeter
     * @return the sector name (integer) of the current sector that
     * this point is in.  If unable to calculate the current sector,
     * returns -1.
     */
    public int calcCurSector(double x, double y) {
        int i, j, r;
        double centerX, centerY;

        double p_x[] = {0,0,0,0,0};
        double p_y[] = {0,0,0,0,0};

        int int_x[] = {0,0,0,0,0};
        int int_y[] = {0,0,0,0,0};


        double cur_x[] = {0,0,0,0,0,0,0};
        double cur_y[] = {0,0,0,0,0,0,0};

        double doub_x[] = {0,0,0,0,0,0,0};
        double doub_y[] = {0,0,0,0,0,0,0};

        // for reference, the coordinates of basic hexagon are:
        //    X[] = {-0.866, -0.866, 0, 0.866, 0.866, 0, -0.866};
        //    Y[] = {-0.5, 0.5, 1.0, 0.5, -0.5, -1.0, -0.5};

        //setup initial x's and y's
        for (i = 0; i < points; i++) {
            cur_x[i] = X[i] * scaler * radius;
            cur_y[i] = Y[i] * scaler * radius;
        }

        //draw sectors and check to see if (x, y) is inside...
        for (i = 0; i < the.sector.length; i++) {
            if (the.sector[i].getName() == -1) {
                break;
            }
/* too much info
            if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {

                the.asNeededPrintln("draw Sect " + i);
                the.asNeededPrintln("   baseX = " + the.sector[i].getBaseX() +
                    " baseY = " + the.sector[i].getBaseY() +
                    " scaler = " + scaler);
            }
*/
            centerX = the.sector[i].getBaseX() * scaler;
            centerY = the.sector[i].getBaseY() * scaler;

            //setup x's and y's based on centerX and Y and direction?

            for (j = 0; j < points; j++) {
                doub_x[j] = centerX + cur_x[j];
                doub_y[j] = centerY + cur_y[j];
            }
// This is too much to print for debugging....
//        the.asNeededPrintln("0: " + (0 + ((i % 3) * 4)) % 6 +
//                   ", 1: " + (1 + ((i % 3) * 4)) % 6 +
//                   ", 2: " + centerX +
//                   ", 3: " + ((5 + ((i % 3) * 4)) % 6));
//        the.asNeededPrintln(" and x and y are: " + x + " " + y +
//                   " and scaled are: " + (x * scaler) + " " +
//                (y * scaler));

                switch (i % 3) {
                case 0 :
                    p_x[0] = doub_x[0];
                    p_x[1] = doub_x[1];
                    p_x[2] = centerX;
                    p_x[3] = doub_x[5];
                    p_x[4] = doub_x[0];

                    p_y[0] = doub_y[0];
                    p_y[1] = doub_y[1];
                    p_y[2] = centerY;
                    p_y[3] = doub_y[5];
                    p_y[4] = doub_y[0];
                    break;
                case 1 :
                    p_x[0] = doub_x[5];
                    p_x[1] = centerX;
                    p_x[2] = doub_x[3];
                    p_x[3] = doub_x[4];
                    p_x[4] = doub_x[5];

                    p_y[0] = doub_y[5];
                    p_y[1] = centerY;
                    p_y[2] = doub_y[3];
                    p_y[3] = doub_y[4];
                    p_y[4] = doub_y[5];
                    break;
                case 2 :
                    p_x[0] = centerX;
                    p_x[1] = doub_x[1];
                    p_x[2] = doub_x[2];
                    p_x[3] = doub_x[3];
                    p_x[4] = centerX;

                    p_y[0] = centerY;
                    p_y[1] = doub_y[1];
                    p_y[2] = doub_y[2];
                    p_y[3] = doub_y[3];
                    p_y[4] = centerY;
                    break;
                default :
                    the.asNeededPrintln("Got an error in doing line segments");
                }

        for (j = 0; j < p_x.length; j++) {
            int_x[j] = (int)p_x[j];
            int_y[j] = (int)p_y[j];
        }
            Polygon p = new Polygon(int_x, int_y, int_x.length);

// This is too much to print for debugging...
//        the.asNeededPrint("Polygon coords: (");
//        for (j = 0; j < 5; j++) {
//        the.asNeededPrint(p_x[j] + ",");
//        }
//        the.asNeededPrint(") (");
//        for (j = 0; j < 5; j++) {
//        the.asNeededPrint(p_y[j] + ",");
//        }
//        the.asNeededPrintln(")");

            // this calculation of inside uses floating point numbers and
            // not integers!!  Therefore, the perimeter check below should
            // only check for vertex points!

            if (calcInside(p_x, p_y, (x * scaler), (y * scaler))) {
                if (DebugLevel.debugging(DebugLevel.INSIDE_DEBUGGING)) {
                    the.asNeededPrintln("*****inside polygon sector: " + i);
                }
                return the.sector[i].getName();
            }
/*          // this is the code which used the polygon inside method
            // which was not sufficient for floating point (x,y) coords.
            if (p.inside((int)((x * scaler)), (int)((y * scaler)))) {
                if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                    the.asNeededPrintln("*****inside polygon sector: " + i);
                }
                System.out.println("p.inside found sec: " +
                    the.sector[i].getNameString() +
                    " and calcInside returns: " +
                    calcInside(p_x, p_y, (x * scaler), (y * scaler)));
                return the.sector[i].getName();
            }
*/
        // before going to next, check the special cases where (x,y)
        // is either a vertex point or is on the lines defining polygon.
        // If it is on the perimeters, this sector is returned - that
        // is, if it's on an edge, the lowest named (i) sector will
        // be the one that is returned.
            if (onPerimeter(x, y, p_x, p_y, p_x.length)) {
//            if (onPerimeter(x, y, int_x, int_y, int_x.length)) {
                if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                    the.asNeededPrint("*****on edge of polygon sector: " + i);
                    the.asNeededPrintln(" onPerimeter " + i);
                    the.asNeededPrintln("   x: " + x + " y: " + y +
                        " i[0]: " + int_x[0] + "," + int_y[0] +
                        " i[1]: " + int_x[1] + "," + int_y[1] +
                        " i[2]: " + int_x[2] + "," + int_y[2] +
                        " i[3]: " + int_x[3] + "," + int_y[3] +
                        " i[4]: " + int_x[4] + "," + int_y[4]);
                    the.asNeededPrintln("sec: " +
                        the.sector[i].getNameString() +
                        " and calcInside returns: " +
                    calcInside(p_x, p_y, (x * scaler), (y * scaler)));
                }
                return the.sector[i].getName();
            }
        }
        if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
            the.asNeededPrintln(
                "****could not determine point's sector location");
        }
        return -1; // error
    }
    /**
     * calcInside() does an inside polygon check using the floating point numbers
     * instead of the integer value inside() function with Polygon class
     * <p>
     * The algorithm used is based on the fact that the sum of all
     * angles inside a triangle should equal 180 degrees.
     * First a sector is divided into two triangles.  The first triangle
     * uses the first three points in the polygon defining the sector
     * (the 0th, 1st and 2nd points).
     * The second triangle is the 2nd, 3rd and 4th points of the polygon.
     * <p>
     * Then using the triangles, all of the inside angles are calculated
     * as shown below.  The sum of A thru F angles should be 180, only
     * if point (x,y) is inside the triangle.
     * <pre>
     *              (x1,y1)
     *                /|\
     *               /A|D\
     *              /  |  \  center is (x,y)
     *             /  / \  \
     *            /E/     \C\
     *    (x2,y2)//_B_____F_\\(x3,y3)
     *
     * </pre>
     * <p>
     * @param x x coordinate of point to examine
     * @param y y coordinate of point to examine
     * @param p_x array containing the x coordinates of the polygon
     * @param p_y array containing the y coordinates of the polygon
     *
     * @see Util#calcTheta
     * @return true if the point is inside the sector polygon defined by
     * p_x and p_y array.  returns false if not inside.
     */
    public boolean calcInside(double [] p_x, double [] p_y,
                                double x, double y) {
        double x1, y1, x2, y2, x3, y3;

        double A, B, C, D, E, F;

        double sum;

        // check first triangle of sector
        x1 = p_x[0]; y1 = p_y[0];
        x2 = p_x[1]; y2 = p_y[1];
        x3 = p_x[2]; y3 = p_y[2];

        A = the.RAD_TO_DEG * Util.calcTheta(x, y, x1, y1, x2, y2);
        B = the.RAD_TO_DEG * Util.calcTheta(x, y, x2, y2, x3, y3);
        C = the.RAD_TO_DEG * Util.calcTheta(x, y, x3, y3, x1, y1);

        D = the.RAD_TO_DEG * Util.calcTheta(x, y, x1, y1, x3, y3);
        E = the.RAD_TO_DEG * Util.calcTheta(x, y, x2, y2, x1, y1);
        F = the.RAD_TO_DEG * Util.calcTheta(x, y, x3, y3, x2, y2);

        sum = A + B + C + D + E + F;

//        System.out.print("A: " + A + "   B: " + B + "   C: " + C);
//        System.out.println("   D: " + D + "   E: " + E + "   F: " + F);
//        System.out.print("x1: " + x1 + " y1:" + y1 +
//                         " x2: " + x2 + " y2:" + y2 +
//                         " x3: " + x3 + " y3:" + y3);
//        System.out.println(" Sum is: " + sum + " and sum == 180 yields: " +
//                            ((sum < 180.0001) && (sum > 179.9999)));

        if ((sum < 180.0001) && (sum > 179.9999)) {
            return true;
        }

        // now check the other triangle of the sector
        x1 = p_x[2]; y1 = p_y[2];
        x2 = p_x[3]; y2 = p_y[3];
        x3 = p_x[4]; y3 = p_y[4];

        A = the.RAD_TO_DEG * Util.calcTheta(x, y, x1, y1, x2, y2);
        B = the.RAD_TO_DEG * Util.calcTheta(x, y, x2, y2, x3, y3);
        C = the.RAD_TO_DEG * Util.calcTheta(x, y, x3, y3, x1, y1);

        D = the.RAD_TO_DEG * Util.calcTheta(x, y, x1, y1, x3, y3);
        E = the.RAD_TO_DEG * Util.calcTheta(x, y, x2, y2, x1, y1);
        F = the.RAD_TO_DEG * Util.calcTheta(x, y, x3, y3, x2, y2);

        sum = A + B + C + D + E + F;

//        System.out.print("A: " + A + "   B: " + B + "   C: " + C);
//        System.out.println("   D: " + D + "   E: " + E + "   F: " + F);
//        System.out.print("x1: " + x1 + " y1:" + y1 +
//                         " x2: " + x2 + " y2:" + y2 +
//                         " x3: " + x3 + " y3:" + y3);
//        System.out.println(" Sum is: " + sum + " and sum == 180 yields: " +
//                            ((sum < 180.0001) && (sum > 179.9999)));

        if ((sum < 180.0001) && (sum > 179.9999)) {
            return true;
        }
        return false;
    }

    /**
     * onPerimeter() checks to see if a point (x,y) is on a vertex point or on
     * the lines defining the sectors
     * <p>
     * @param x x coordinate of point to examine
     * @param y y coordinate of point to examine
     * @param p_x array containing the x coordinates of the polygon
     * @param p_y array containing the y coordinates of the polygon
     * @param s size of polygon arrays.
     *
     * @return true if the point (x,y) is on the perimeter of polygon
     * defined by
     * p_x and p_y array.  returns false if not on edges.
     */
    public boolean onPerimeter(double x, double y, double [] p_x, double [] p_y,
                    int s) {
    double m;
    double b;
    double num;
    double den;

    double sx = x * scaler;
    double sy = y * scaler;

    for (int i = 0; i < s; i++) {

        // first check to see if point is at a vertex point
        if (Util.isEquiv(sx, p_x[i]) && Util.isEquiv(sy, p_y[i])) {//vertex point
            if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                the.asNeededPrintln("Location is on a vertex point");
            }
            return true;
        }

        // now check line from p[i] to p[i+1]
        // use form mx + b = y where m=delta y / delta x
        // and b = y - mx
        num = (p_y[(i + 1) % s] - p_y[i]);
        den = (p_x[(i + 1) % s] - p_x[i]);

        if (Util.isEquiv(den, 0)) { // vertical line defined since denominator == 0
            if (Util.isEquiv(sx, p_x[i])) {  // see if x is on this line
                // now check if sx is on segment
                if (num > 0) {
                    if ((p_y[(i + 1) % s] > sy) && (sy > p_y[i])) {
                        if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                            the.asNeededPrintln("Location on a vertical line");
                        }
                        return true;
                    }
                } else {
                    if ((p_y[(i + 1) % s] < sy) && (sy < p_y[i])) {
                        if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                            the.asNeededPrintln("Location on a vertical line");
                        }
                        return true;
                    }
                }
            }
        } else {
            m = num / den;

            b = p_y[i] - (m * p_x[i]);

            if (Util.isEquiv(((m * sx) + b), sy)) {
            // need to check if (sx,sy) is on segment
                if (num > 0) {
                    if ((p_y[(i + 1) % s] > sy) && (sy > p_y[i])) {
                        if (den > 0) {
                            if ((p_x[(i + 1) % s] > sx) && (sx > p_x[i])) {
                                if (DebugLevel.debugging(
                                        DebugLevel.USER_DEBUGGING)) {
                                    the.asNeededPrintln(
                                        "Location on a slanted line");
                                }
                                return true;
                            }
                        } else {
                            if ((p_x[(i + 1) % s] < sx) && (sx < p_x[i])) {
                                if (DebugLevel.debugging(
                                        DebugLevel.USER_DEBUGGING)) {
                                    the.asNeededPrintln(
                                        "Location on a slanted line");
                                }
                                return true;
                            }
                        }
                    }
                } else {
                    if ((p_y[(i + 1) % s] < sy) && (sy < p_y[i])) {
                        if (den > 0) {
                            if ((p_x[(i + 1) % s] > sx) && (sx > p_x[i])) {
                                if (DebugLevel.debugging(
                                        DebugLevel.USER_DEBUGGING)) {
                                    the.asNeededPrintln(
                                        "Location on a slanted line");
                                }
                                return true;
                            }
                        } else {
                            if ((p_x[(i + 1) % s] < sx) && (sx < p_x[i])) {
                                if (DebugLevel.debugging(
                                        DebugLevel.USER_DEBUGGING)) {
                                    the.asNeededPrintln(
                                        "Location on a slanted line");
                                }
                                return true;
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
    }

    /**
     * This method will take a user's visible sectors and try to draw
     * them on the map as transparent yellow, gray or green sectors.
     * This algoritm uses XOR-ing of the background or other objects
     * in the area with the color of the sector being drawn so that
     * everything is still visible.
     *
     * @param u user that will be examined for its visible sectors.
     */
    public void showVisibleSectors(User u, Graphics g) {
        int [] vis;
        int [] p_x = {0,0,0,0,0};
        int [] p_y = {0,0,0,0,0};
        vis = u.getIntNamesForVisibleSectors();
        if (vis[0] == -1) {
            System.err.println("No values found in Visible Sectors String");
            return;
        }
        // now search line segment array for lines used for each segment
        // and grab from that line a point.
		drawFilledSectors(vis, g, Color.darkGray);  // show in light gray
		int [] ant = new int [1];
		if(u.getCurAntenna()!=-1)
		{
			ant[0] = u.getCurAntenna();
			drawFilledSectors(ant, g, Color.darkGray);  // shut off what was set
			drawFilledSectors(ant, g, (Color.blue).brighter());
                                    // show in yellow (blue is !yellow)
		}
		else
		{
			ant[0] = u.getCurSector();
			drawFilledSectors(ant, g, Color.darkGray);  // shut off what was set
			drawFilledSectors(ant, g, Color.red);
                                    // show in yellow (blue is !yellow)
		}
    }

    /**
     * This method is called by showVisibleSectors method to do the
     * actual polygon drawing.
     * <p>
     * @param vis is an array of integers corresponding to array indices into
     * sectors array.
     *
     * @param color is the color which is XOR-ed so users can still be seen.
     * This color must be the inverse of color desired.  This produces
     * different colors depending on what architecture the program is
     * being run on!
     */
    public void drawFilledSectors(int vis[], Graphics g, Color color) {
        int i, j, r;
        double centerX, centerY;

        double p_x[] = {0,0,0,0,0};
        double p_y[] = {0,0,0,0,0};

        int int_x[] = {0,0,0,0,0};
        int int_y[] = {0,0,0,0,0};


        double cur_x[] = {0,0,0,0,0,0,0};
        double cur_y[] = {0,0,0,0,0,0,0};

        double doub_x[] = {0,0,0,0,0,0,0};
        double doub_y[] = {0,0,0,0,0,0,0};

        // for reference, the coordinates of basic hexagon are:
        //    X[] = {-0.866, -0.866, 0, 0.866, 0.866, 0, -0.866};
        //    Y[] = {-0.5, 0.5, 1.0, 0.5, -0.5, -1.0, -0.5};

        if (vis[0] == -1) {
            return; // no valid data in array
        }
        //setup initial x's and y's
        for (i = 0; i < points; i++) {
            cur_x[i] = (int)((X[i] * scaler * radius));
            cur_y[i] = (int)((Y[i] * scaler * radius));
        }

        //draw sectors and check to see if (x, y) is inside...
        for (i = 0; i < vis.length; i++) {
            if (the.sector[vis[i]].getName() == -1) {
                break;
            }

            centerX = the.sector[vis[i]].getBaseX() * scaler + inset-MapXoffset+Xoffset;
            centerY = the.sector[vis[i]].getBaseY() * scaler + inset-MapYoffset+Yoffset;

            //setup x's and y's based on centerX and Y and direction?

            for (j = 0; j < points; j++) {
        doub_x[j] = centerX + cur_x[j];
        doub_y[j] = centerY + cur_y[j];
            }

                switch (vis[i] % 3) {
                case 0 :
                    p_x[0] = doub_x[0];
                    p_x[1] = doub_x[1];
                    p_x[2] = centerX;
                    p_x[3] = doub_x[5];
                    p_x[4] = doub_x[0];

                    p_y[0] = doub_y[0];
                    p_y[1] = doub_y[1];
                    p_y[2] = centerY;
                    p_y[3] = doub_y[5];
                    p_y[4] = doub_y[0];
                    break;
                case 1 :
                    p_x[0] = doub_x[5];
                    p_x[1] = centerX;
                    p_x[2] = doub_x[3];
                    p_x[3] = doub_x[4];
                    p_x[4] = doub_x[5];

                    p_y[0] = doub_y[5];
                    p_y[1] = centerY;
                    p_y[2] = doub_y[3];
                    p_y[3] = doub_y[4];
                    p_y[4] = doub_y[5];
                    break;
                case 2 :
                    p_x[0] = centerX;
                    p_x[1] = doub_x[1];
                    p_x[2] = doub_x[2];
                    p_x[3] = doub_x[3];
                    p_x[4] = centerX;

                    p_y[0] = centerY;
                    p_y[1] = doub_y[1];
                    p_y[2] = doub_y[2];
                    p_y[3] = doub_y[3];
                    p_y[4] = centerY;
                    break;
                default :
                    the.asNeededPrintln("Got an error in doing line segments");
                }

        for (j = 0; j < p_x.length; j++) {
            int_x[j] = (int)p_x[j];
            int_y[j] = (int)p_y[j];
        }
            //g.setColor(color1);
            g.setXORMode(color);
            g.fillPolygon(int_x, int_y, int_x.length); //DRAW IT!!
        }

    }
}

