/*
 * MapPanel class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/29/97
 * Date Last Revised: 4/12/97
 */
import java.awt.*;

/**
 * This class is the center portion of UTMOST2.  In it will be the
 * MapArea, the Legend and the 4 buttons for showing or hiding additional
 * features on the Map.
 * <p>
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see MapArea
 * @see UTMOST2
 * @see UserStats
 * @see ModelSetup
 */
public class MapPanel extends Panel {

    private UTMOST2 the;
    private MapArea theMap;
    private Panel buttonPanel;
    private Panel east,center;
    private Legend legend;

    private Button showAntAssign = new Button("Show All Antennae");
    private Button showAntChanges = new Button("Show Changed Antennae");
    private Button showPwrChanges = new Button("Show Changed Power");
    private Button showSectorNames = new Button("Show Sector Names");
    private Label dirText = new Label();
	private Scrollbar hbar;
	private Scrollbar vbar;
//    private ComponentScroller scroller;

// Constructors
    public MapPanel() {
        // use default FlowLayout
    }
    /**
     * Constructor which will create a MapPanel object.
     *
     * @param u is the UTMOST2 object which called this constructor
     */
    public MapPanel(UTMOST2 u) {
        the = u;
        theMap = the.map;
        legend = the.legend;

		
		hbar = new Scrollbar(Scrollbar.HORIZONTAL,0,10,0,1000);
		vbar = new Scrollbar(Scrollbar.VERTICAL,0,10,0,1000);
		hbar.setBackground(Color.lightGray);
		vbar.setBackground(Color.lightGray);
		

        buttonPanel = new Panel();
//        scroller = new ComponentScroller(theMap);

        // Setup the Buttons for the East side
        buttonPanel.setLayout(new GridLayout(5,1,5,5));
        buttonPanel.add(showAntAssign);
        buttonPanel.add(showAntChanges);
        buttonPanel.add(showPwrChanges);
        buttonPanel.add(showSectorNames);
        buttonPanel.add(dirText);

        east = new Panel();
        east.setLayout(new BorderLayout(5,5));
        east.add("North", legend);
        east.add("Center", buttonPanel);

		center = new Panel();
		center.setLayout(new BorderLayout(1,1));
		center.add("Center",theMap);
		center.add("East",vbar);
		center.add("South",hbar);
        setLayout(new BorderLayout(5,5));
        add("East", east);
        add("Center", center);
		if(the.drawImage){
			setHScrollbarVisible(1); //show scrollbara
			setVScrollbarVisible(1);
		}
		else{
			setHScrollbarVisible(0); //hide scrollbara
			setVScrollbarVisible(0);
		}
    }

    public void setUTMOST2(UTMOST2 u) {
        the = u;
    }

    public void setDirText(String s) {
        dirText.setText(s);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        if (evt.target == showAntAssign) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Show Antenna Assignments Button Pressed");
            }
            showAntAssignAction();
        }
        if (evt.target == showAntChanges) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Show Antenna Changes Button Pressed");
            }
            showAntChangesAction();
        }
        if (evt.target == showPwrChanges) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Show Power Changes Button Pressed");
            }
            showPwrChangesAction();
        }
        if (evt.target == showSectorNames) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Show Sector Names Button Pressed");
            }
            showSectorNamesAction();
        }
		return super.action(evt, arg);
    }

	public boolean handleEvent(Event event){
		if(event.id == Event.SCROLL_LINE_DOWN ||
			event.id == Event.SCROLL_PAGE_UP ||
			event.id == Event.SCROLL_PAGE_DOWN ||
			event.id == Event.SCROLL_ABSOLUTE ){
			theMap.setMapOffsets(hbar.getValue(),vbar.getValue());
			return true;
		}
		return super.handleEvent(event);
	}

	public void setHScrollbarVisible(int x){
		if(x == 0){
			hbar.setValues(hbar.getValue(),0,hbar.getMinimum(),hbar.getMaximum());		
		}
		else{
			hbar.setValues(hbar.getValue(),1,hbar.getMinimum(),hbar.getMaximum());
		}
	}

	public void setVScrollbarVisible(int x){
		if(x == 0){
			vbar.setValues(vbar.getValue(),0,vbar.getMinimum(),vbar.getMaximum());
		}
		else{
			vbar.setValues(vbar.getValue(),1,vbar.getMinimum(),vbar.getMaximum());
		}
	}

	public void setHScrollbarValue(int x){
		hbar.setValue(hbar.getValue() + x); 
	}

	public void setVScrollbarValue(int x){
		vbar.setValue(vbar.getValue() + x); 
	}

    public void showAntAssignAction() {
        if (theMap.getShouldShowAllAntennae()) {
            theMap.setShouldShowAllAntennae(false);
            showAntAssign.setLabel("Show All Antennae");
        } else {
            theMap.setShouldShowAllAntennae(true);
            showAntAssign.setLabel("Hide All Antennae");
        }
        theMap.updateMap();
    }

    public void showAntChangesAction() {
        if (theMap.getShouldShowChangedAntennae()) {
            theMap.setShouldShowChangedAntennae(false);
            showAntChanges.setLabel("Show Changed Antennae");
        } else {
            theMap.setShouldShowChangedAntennae(true);
            showAntChanges.setLabel("Hide Changed Antennae");
        }
        theMap.updateMap();
    }

    public void showPwrChangesAction() {
        if (theMap.getShouldShowChangedPower()) {
            theMap.setShouldShowChangedPower(false);
            showPwrChanges.setLabel("Show Changed Power");
        } else {
            theMap.setShouldShowChangedPower(true);
            showPwrChanges.setLabel("Hide Changed Power");
        }
        theMap.updateMap();
    }

    public void showSectorNamesAction() {
        if (theMap.getShouldShowSectors()) {
            theMap.setShouldShowSectors(false);
            showSectorNames.setLabel("Show Sector Names");
        } else {
            theMap.setShouldShowSectors(true);
            showSectorNames.setLabel("Hide Sector Names");
        }
        theMap.updateMap();
    }
}

