/*
 * PowCtrlPanel class
 *
 * Author: Jonas Hedlind
 * Date Started: 06/97
 * Date Last Revised: 07/97
 */
import java.awt.*;
/**
 * 
 * <p>
 * @author      Jonas Hedlind
 * @version RACEWIN 1.0
 * @see VariablesPanel
 * @see FilesPanel
 * @see SectorPanel
 * @see PropertySettings
 */
public class PowCtrlPanel extends Panel 
{
	private Panel south;
	private Panel main;
	private Panel center;
	private Panel north;
	
	private Label PanelTitle = new Label("Power Control Variables");
	private Label PowCtrlLabel = new Label("Type Of Power Ctrl");
	private Choice PowerControlChoice =  new Choice();
	private Label         FadingLabel = new Label("Fading Model");
	private Choice        FadingChoice = new Choice();
	
    
	private Label         MaxPowerLabel = new Label("Max Power:");
	private TextField     MaxPowerValueText = new TextField("1.0");
	private Label         MinPowerLabel = new Label("Min Power:");
	private TextField     MinPowerValueText = new TextField("1e-8");
	
	

	private TextField     NoiseValueText = new TextField("1e-14");
	private Label         NoiseLabel = new Label("Noise:");
	private TextField     CIRValueText = new TextField("0.05");
	private Label         CIRLabel = new Label("CIR:");
	private TextField     HandoffValueText = new TextField("0.25");
	private Label         HandoffLabel = new Label("Handoff Threshold");
	private TextField     MaxRepValueText = new TextField("30");
	private Label         MRepsLabel = new Label("Maximum Iterations");

	//private Label		  RacesimLabel = new Label ("Generate Boundary Files");
	private Checkbox	  RacesimCheck = new Checkbox();

	private PropertySettings parent;
	private UTMOST2 the;
    /**
     * Constructor which will create a PowCtrlPanel object.
     *
     * @param p is the PropertySettings panel object which called this
     * constructor
     */
    public PowCtrlPanel(PropertySettings p) 
	{
        parent = p;
		the = parent.getUTMOST2();
		
		north = new Panel();
		north.setLayout(new GridLayout(1,4));
		north.add(PowCtrlLabel);
		PowerControlChoice.addItem("No PowerControl");
		PowerControlChoice.addItem("POCAT");
        PowerControlChoice.addItem("File Interaction");
		north.add(PowerControlChoice);
		north.add(FadingLabel);
		FadingChoice.addItem("Single Step");
		FadingChoice.addItem("Multiple Step");
		north.add(FadingChoice);

		center = new Panel();
		center.setLayout(new GridLayout(3,4,2,10));
		center.add(MaxPowerLabel);
		center.add(MaxPowerValueText);
		center.add(NoiseLabel);
		center.add(NoiseValueText);
		center.add(MinPowerLabel);
		center.add(MinPowerValueText);
		center.add(CIRLabel);
		center.add(CIRValueText);
		center.add(HandoffLabel);
		center.add(HandoffValueText);
		center.add(MRepsLabel);
		center.add(MaxRepValueText);

		south = new Panel();
		//south.setLayout();
		RacesimCheck.setLabel("Generate Boundary Files");
		south.add(RacesimCheck);
		//south.add(RacesimLabel);
		

		main = new Panel();
		main.setLayout(new BorderLayout(5,5));
		main.add("North",north);
		main.add("Center",center);
		main.add("South",south);
		
		add("North",PanelTitle);
		add("South",main);

		PowerControlChoice.select(1); //Default Pocat
		FadingChoice.select(0);
		RacesimCheck.setState(true);

    }

	public int getSelctedPowerControl() 
	{
        return (PowerControlChoice.getSelectedIndex());
    }

	public int getFadingModel() 
	{
        return (FadingChoice.getSelectedIndex());
    }

    public void setParent(PropertySettings p)
	{
        parent = p;
    }

    public void setPowCtrlType(int i) 
	{
        (parent.getUTMOST2()).PowerControlType = i;
    }

	private double getMaxPower(){
		double v = (Double.valueOf(MaxPowerValueText.getText())).doubleValue();
		if (v > 0 && v < 20 )
			return v;
		else{
			MaxPowerValueText.setText("1");
			return (1);
		}
	}

	private double getMinPower(){
		double v = (Double.valueOf(MinPowerValueText.getText())).doubleValue();
		if (v > 0 && v < getMaxPower() )
			return v;
		else{
			MinPowerValueText.setText("1e-8");
			return (1e-8);
		}
	}

	private double getNoise(){
		double v = (Double.valueOf(NoiseValueText.getText())).doubleValue();
		if (v > 0  )
			return v;
		else{
			NoiseValueText.setText("1e-14");
			return (1e-14);
		}
	}

	private double getHandoff(){
		double v = (Double.valueOf(HandoffValueText.getText())).doubleValue();
		if (v > 0  && v<1)
			return v;
		else{
			HandoffValueText.setText("0.25");
			return (0.25);
		}
	}

	private double getCIR(){
		double v = (Double.valueOf(CIRValueText.getText())).doubleValue();
		if (v > 0  && v<1)
			return v;
		else{
			CIRValueText.setText("0.05");
			return (0.05);
		}
	}

	private int getMaxReps(){
		int v = Integer.parseInt(MaxRepValueText.getText());
		if (v > 0 )
			return v;
		else{
			MaxRepValueText.setText("30");
			return (30);
		}
	}

	

   
   /*
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) 
	{
		//setPowCtrlType(PowerControlChoice.getSelectedIndex());
		updateValues();
        return true;
    }

    public void updateValues() 
	{
        setPowCtrlType(PowerControlChoice.getSelectedIndex());
		the.PowerControlSetting.setMaxPower(getMaxPower());
		the.PowerControlSetting.setMinPower(getMinPower());
		the.PowerControlSetting.setMaxReps(getMaxReps());
		the.PowerControlSetting.setNoise(getNoise());
		the.PowerControlSetting.setFadingModel(getFadingModel());
		the.PowerControlSetting.setHandoffThreshold(getHandoff());
		the.PowerControlSetting.setCir(getCIR());
		the.generateBoundary=RacesimCheck.getState();
	}
}

