/*
 * PropertySettings class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/12/97
 * Date Last Revised: 4/12/97
 */

/**
 * PropertySettings is a class which allows variables used in UTMOST2 model
 * to be changed.  (Cannot use the word Properties since this is a java class
 * in the JDK... not worth making sure the CLASSPATH is correct).
 * <p>
 * This class does not function completely.  It merely provides the
 * hooks to perform some parameterization of some of the variables used
 * in the model.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see VariablesPanel
 * @see FilesPanel
 * @see UsersPanel
 * @see SectorsPanel
 */

import java.awt.*;

public class PropertySettings extends Dialog {

    private Button variablesButton = new Button("Variables");
    private Button filesButton = new Button("Files");
    private Button usersButton = new Button("Users");
    private Button sectorsButton = new Button("Sectors");
	private Button PowCtrlButton = new Button("PowerCtrl");
	private Button imageButton = new Button("Image");
    private Button closeButton = new Button("Close");

    private Panel tabs;
    private Panel cards;
    private CardLayout layout;

    private VariablesPanel variablesPanel;
    private FilesPanel filesPanel;
    private UsersPanel usersPanel;
    private SectorsPanel sectorsPanel;
	private PowCtrlPanel PowCtrlPanel;
	private ImagePanel imagePanel;

    private UTMOST2 the;

	

    /**
     * Constructor which will create a PropertySettings object.
     *
     * @param t is the Frame object to which this dialog belongs.
     */
    public PropertySettings(Frame t) {
        super(t, "Properties for RACEWIN Model", true);

        the = (UTMOST2) t;

        tabs = new Panel();
        tabs.add(variablesButton);
        tabs.add(filesButton);
        tabs.add(usersButton);
        tabs.add(sectorsButton);
		tabs.add(PowCtrlButton);
		tabs.add(imageButton);
        tabs.add(closeButton);

        add("North", tabs);

        cards = new Panel();
        layout = new CardLayout();
        cards.setLayout(layout);

        variablesPanel = new VariablesPanel(this);
        filesPanel = new FilesPanel(this);
        usersPanel = new UsersPanel(this);
        sectorsPanel = new SectorsPanel(this);
		PowCtrlPanel = new PowCtrlPanel(this);
		imagePanel = new ImagePanel(this);

        cards.add("Variables", variablesPanel);
        cards.add("Files", filesPanel);
        cards.add("Users", usersPanel);
        cards.add("Sectors", sectorsPanel);
		cards.add("PowerCtrl",PowCtrlPanel);
		cards.add("Image",imagePanel);

        add("Center", cards);

        resize(insets().left + insets().right + 650,
                insets().top + insets().bottom + 450);

    }

    public UTMOST2 getUTMOST2() {
        return the;
    }

    public synchronized void show() {
        move(50, 50);
        super.show();
    }

    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event evt) {
        if (evt.id == Event.WINDOW_DESTROY) {
            hide();
            // dispose();
            return true;
        }
        return super.handleEvent(evt);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        variablesPanel.updateValues();
        filesPanel.updateValues();
        sectorsPanel.updataValues();
        usersPanel.updateValues();
		PowCtrlPanel.updateValues();
		imagePanel.updateValues();
        if (evt.target instanceof Component &&
                ((Component)evt.target).getParent().equals(tabs)) {
            if (arg.equals("Close")) {
                hide();
                return super.action(evt, arg);
                // System.exit(0);
            }
//            the.asNeededPrintln(
//        "Properties: Inside action() before layout.show");
            layout.show(cards, (String)arg);
        } else {
            return super.action(evt, arg);
        }
        return true;
    }
    /**
     * This is just a test main method.  It is not used by UTMOST2.
     */
    public static void main(String [] args) {
        PropertySettings myProp = new PropertySettings(new Frame());
        myProp.show();
    }

}
