/*
 * QuitBox Dialog
 *
 * Author: M. Heidi McClure
 * Date Started: 9/1/96
 * Date Last Revised: 4/12/97
 *
 */

import java.awt.*;

/**
 * This class just displays the quit
 * UTMOST2 dialog.
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 */
class QuitBox extends Dialog {

    /**
     * Constructor which will display the Quit dialog box.
     *
     * @param parent is the caller of this class.
     */
    public QuitBox(Frame parent) {

        super(parent, "Quit Application?", true);
        setResizable(false);

        setLayout(null);
        addNotify();
        resize( insets().left + insets().right + 261,
                insets().top + insets().bottom + 72);
        yesButton=new Button("Yes");
        add(yesButton);
        yesButton.reshape(insets().left + 68,insets().top + 10,46,23);
        noButton=new Button("No");
        add(noButton);
        noButton.reshape(insets().left + 135,insets().top + 10,47,23);
    }

    public synchronized void show() {
        Rectangle bounds = getParent().bounds();
        Rectangle abounds = bounds();

        move(   bounds.x + (bounds.width - abounds.width)/ 2,
                bounds.y + (bounds.height - abounds.height)/2);

        super.show();
    }

    public synchronized void wakeUp() {
        notify();
    }

    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     * This handleEvent has button actions in it... these should be in
     * an action method.  This may be why under some OS's the buttons
     * don't behave correctly? in these dialogs
     */
   public boolean handleEvent(Event event) {
        if (event.id == Event.ACTION_EVENT && event.target == noButton) {
            clickedNoButton();
            return true;
        } else if (event.id == Event.ACTION_EVENT && event.target == yesButton) {
            clickedYesButton();
            return true;
        } else if (event.id == Event.WINDOW_DESTROY) {
            hide();
            return true;
        }
        return super.handleEvent(event);
    }

    Button yesButton;
    Button noButton;

    public void clickedYesButton() {
        System.exit(0);
    }
    public void clickedNoButton() {
        handleEvent(new Event(this, Event.WINDOW_DESTROY, null));
    }
}
