/*
 * Sector class
 *
 * Author: M. Heidi McClure
 * Date Started: 10/31/96
 * Date Last Revised: 4/12/97
 */

/**
 * This is the sector class.  Sectors are the 1/3 sections of the
 * hexagon cells.
 * <p>
 * Sector information is read in from a file.  The attributes of a
 * Sector come directly from the file.  Sectors have the following
 * attributes:
 * <p>
 * name<br>
 * cornerName<br>
 * baseX<br>
 * baseY<br>
 * direction<br>
 * radius<br>
 * range<br>
 *
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 */
public class Sector {

    private int name;
    private int cornerName;
    private double baseX;
    private double baseY;
    private double direction;
    private double radius;
    private double range;

	private int AssignedUsers=0;
	private double ReceivedPower=0;

    /**
     * Default Constructor which will create a Sector object with all
     * attributes set to -1;
     */
    public Sector() {
        name = -1;
        cornerName = -1;
        baseX = -1;
        baseY = -1;
        direction = -1;
        radius = -1;
        range = -1;
    }
    /**
     * Constructor which will create a Sector object.
     * <p>
    `* @param n is the name
     * @param c is the cornerName
     * @param x is the baseX
     * @param y is the baseY
     * @param d is the direction
     * @param rad is the radius
     * @param rng is the range
     */
    public Sector(int n, int c, double x, double y, double d,
            double rad, double rng) {
        name = n;
        cornerName = c;
        baseX = x;
        baseY = y;
        direction = d;
        radius = rad;
        range = rng;
    }
    /**
     * This method will create a string displaying values of all
     * variables of this Sector.
     * @return the formatted string for System.out.println, for example
     */
    public String toString() {
    return new String("strname: " + getNameString() +
            " name: " + name + ", cornerName: " + cornerName +
            ", baseX: " + baseX + ", baseY: " + baseY +
            ",\n\tdirection: " + direction +
            ", sig radius: " + radius + ", sig range: " + range);
    }
    public void setName(int i) {
        name = i;
    }
    public int getName() {
        return name;
    }

    /**
     * @return the human readable sector name like 4C or 2B
     */
    public String getNameString() {
    int cell;
        int sect;
        StringBuffer ret = new StringBuffer();
        cell = (1 + name / 3) + (int)'0';
        sect = (name % 3) + (int)'A';
        ret.append((char)cell);
        ret.append((char)sect);
        return(ret.toString());
    }

    public void setCornerName(int i) {
        cornerName = i;
    }
    public int getCornerName() {
        return cornerName;
    }

    public void setBaseX(double x) {
        baseX = x;
    }
    public double getBaseX() {
        return baseX;
    }

    public void setBaseY(double y) {
        baseY = y;
    }
    public double getBaseY() {
        return baseY;
    }
/*
    public void setDirection(double d) {
        direction = d;
    }
*/
    public double getDirection() {
        return direction;
    }

    public void setRadius(double r) {
        radius = r;
    }
    public double getRadius() {
        return radius;
    }

    public void setRange(double r) {
        range = r;
    }
    public double getRange() {
        return range;
    }

	public int getAssignedUsers(){
		return AssignedUsers;
	}
	public void setAssignedUsers(int u){
		AssignedUsers=u;
	}

	public double getReceivedPower(){
		return ReceivedPower;
	}
	public void setReceivedPower(double p){
		ReceivedPower = p;
	}

}
