/*
 * PropertySettings class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/12/97
 * Date Last Revised: 4/12/97
 */

/**
 * PropertySettings is a class which allows variables used in UTMOST2 model
 * to be changed.  (Cannot use the word Properties since this is a java class
 * in the JDK... not worth making sure the CLASSPATH is correct).
 * <p>
 * This class does not function completely.  It merely provides the
 * hooks to perform some parameterization of some of the variables used
 * in the model.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see VariablesPanel
 * @see FilesPanel
 * @see UsersPanel
 * @see SectorsPanel
 */

import java.awt.*;

public class SectorInfo extends Dialog {

    private Panel buttons,center,data;
    private UTMOST2 the;
	private Button [] sectorButton;
	private Button exitButton = new Button("Exit");

	private Label title = new Label("Information for Sector    ");
	private Label userNuberLabel = new Label("Number of Users:");
	private Label totalPowerLabel = new Label("Received Power:");
	private Label numberOfUsers = new Label("0    ");
	private Label toatalPower = new Label("0             ");

	

    /**
     * Constructor which will create a PropertySettings object.
     *
     * @param t is the Frame object to which this dialog belongs.
     */
    public SectorInfo(Frame t) {
        super(t, "Sector Information", true);

        the = (UTMOST2) t;

        buttons = new Panel();

		sectorButton = new Button[the.sector.length];
		
		for(int i=0; i < the.sector.length;i++){
			sectorButton[i] = new Button(Util.getSectorString(i));
			buttons.add(sectorButton[i]);
		}
        
		data = new Panel();
		data.setLayout(new GridLayout(2,3,5,5));
		data.add(userNuberLabel);
		data.add(numberOfUsers);
		data.add(new Label(" "));
		data.add(totalPowerLabel);
		data.add(toatalPower);
		data.add(new Label(" "));
		center = new Panel();
		center.setLayout(new BorderLayout(5,5));
		center.add("North",title);
		center.add("Center",data);

		//exitButton.resize(70,30);

		add("North", buttons);
		add("Center",center);
		add("South",exitButton);
		
        resize(insets().left + insets().right + 750,
                insets().top + insets().bottom + 400);
    }

	public void showSectorInfo(String name){
		title.setText("Information for Sector " +name);
		int n = Util.getSectorInt(name);
		numberOfUsers.setText(new Integer(the.sector[n].getAssignedUsers()).toString());
		toatalPower.setText(new Double(the.sector[n].getReceivedPower()).toString());
	}

    public UTMOST2 getUTMOST2() {
        return the;
    }

    public synchronized void show() {
        move(50, 50);
        super.show();
    }

    /**
     * handleEvent is for handling mouse, keystroke, Window, ScrollBar,
     * List and some action events.  Geary, pg. 69
     */
    public boolean handleEvent(Event evt) {
        if (evt.id == Event.WINDOW_DESTROY) {
            hide();
            // dispose();
            return true;
        }
        return super.handleEvent(evt);
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        
        if (evt.target instanceof Component &&
                ((Component)evt.target).getParent().equals(buttons)) {
			String label = (String) arg;
			//int n = Integer.parseInt(label);
			showSectorInfo(label); 
            return super.action(evt, arg);
                // System.exit(0);
        }
        if (evt.target == exitButton){
			hide();
			return super.action(evt, arg);
		}
		
		return super.action(evt, arg);
    }

	public void calcSectorInfo(){
		
		for(int i=0;i<the.sector.length;i++){
			double p=0;
			for(int j=0;j< the.map.getNumUsers();j++){
				if(the.users[j].IsVisible(i)==1){
					p+=the.users[j].getPathloss(i)*the.users[j].getCurPower();
				}
				the.sector[i].setReceivedPower(p);
			}
		}
		showSectorInfo("1A");
	}
}
