/*
 * User class
 *
 * Author: M. Heidi McClure
 * Date Started: 9//96
 * Date Last Revised: 4/12/97
 */

import java.awt.*;
import java.util.*; // for class Random and StringTokenizer
import java.lang.*; // for sin and cos

/**
 * This is the user class.
 * <p>
 * User information is generated randomly based on model parameters.
 * Users have the following
 * attributes:
 * <p>
 * name<br>
 * isCalling<br>
 * xLocation array<br>
 * yLocation array<br>
 * cur index into arrays above<br>
 * <br>
 * prevPrevXLocation<br>
 * prevPrevYLocation<br>
 * prevXLocation<br>
 * prevYLocation<br>
 * curXLocation<br>
 * curYLocation<br>
 * <br>
 * curPower<br>
 * prevPower<br>
 * <br>
 * curMoveSpeed<br>
 * prevMoveSpeed<br>
 * curDirection<br>
 * prevDirection<br>
 * <br>
 * curSector<br>
 * prevSector<br>
 * curAntenna<br>
 * prevAntenna<br>
 * <br>
 * curVisibleSectors<br>
 * <br>
 * xDext<br>
 * yDest<br>
 * <br>
 * xBoundaryCross<br>
 * yBoundaryCross<br>
 * <br>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see UTMOST2
 */
public class User {

    public static final int DRIVING = 0; // user is moving, but phone not on.
    public static final int ROAMING = 1; // Phone on, but not calling.
    public static final int CALLING = 2; // Phone on and user calling
    private int name;
    private int isCalling;  // only allow  DRIVING, ROAM, CALLING
    private double curPower;
    private double prevPower;

    private double [] xLocation = new double [10]; // these are circular arrays
    private double [] yLocation = new double [10];
    private int cur; // cur location index into above arrays.

    private double prevPrevXLocation;  // in future may need all the other params,
    private double prevPrevYLocation;  // too, like prevPrevMoveSpeed...

    private double prevXLocation;
    private double prevYLocation;
    private int prevMoveSpeed;
    private double prevMoveDirection;
    private int prevSector;
    private int prevAntenna;

    private double curXLocation;
    private double curYLocation;
    private int curMoveSpeed;
    private double curMoveDirection;
    private int curSector;
    private int curAntenna;
    private String curVisibleSectors;
	//Jonas
	private int [] prevHandoff = new int [6];
	private int [] curHandoff = new int [6];
	private int prevNumber; // Prev Number of antennas
	private int curNumber;

	private int [] VisibleSectors = new int [21]; /*Max 7 Cells */ 
	private double [] Pathloss = new double [7]; 
	private int [] possibleAnt = new int[7]; //BSs that the user can connect to
	//
    private double xDest;
    private double yDest;

    private double xBoundaryCross;
    private double yBoundaryCross;

    private UTMOST2 the;

    /**
     * Constructor which will create a User object.  All attributes
     * are set as follows:
     * name = i (passed in parameter)<br>
     * isCalling = DRIVING<br>
     * xLocation array = 0 for all elements<br>
     * yLocation array = 0 for all elements<br>
     * cur index into arrays above = 0<br>
     * <br>
     * prevPrevXLocation = 0<br>
     * prevPrevYLocation = 0<br>
     * prevXLocation = 0<br>
     * prevYLocation = 0<br>
     * curXLocation = 0<br>
     * curYLocation = 0<br>
     * <br>
     * curPower = 0<br>
     * prevPower = 0<br>
     * <br>
     * curMoveSpeed = 0<br>
     * prevMoveSpeed = 0<br>
     * curDirection = -1<br>
     * prevDirection = -1<br>
     * <br>
     * curSector = -1<br>
     * prevSector = -1<br>
     * curAntenna = -1<br>
     * prevAntenna = -1<br>
     * <br>
     * curVisibleSectors (not set here)<br>
     * <br>
     * xDext = 0<br>
     * yDest = 0<br>
     * <br>
     * xBoundaryCross = 0<br>
     * yBoundaryCross = 0<br>
     * <br>
     * <p>
    `* @param t is the calling UTMOST2 model
     * @param i is the user name
     */
    public User(UTMOST2 t, int i) {
        name = i;
        isCalling = DRIVING;
        the = t;
        curPower = 0.0;
        prevPower = 0.0;

        cur = 0;
        for (int j = 0; j < xLocation.length; j++) {
            xLocation[j] = 0;
            yLocation[j] = 0;
        }
        prevXLocation = 0;
        prevYLocation = 0;
        prevPrevXLocation = 0;
        prevPrevYLocation = 0;
        curXLocation = 0;
        curYLocation = 0;
        curMoveSpeed = 0;
        prevMoveSpeed = 0;
        curMoveDirection = -1;
        prevMoveDirection = -1;
        xDest = 0;
        yDest = 0;
        prevSector = -1;
        curSector = -1;
        curAntenna = -1;
        prevAntenna = -1;

        xBoundaryCross = 0;
        yBoundaryCross = 0;
    }

    /**
     * This method will create a string displaying values of all
     * variables of this User.
     * @return the formatted string for System.out.println, for example
     */
    public String toString() {
        return ("name: " + name +
                " \n\t x: " + curXLocation +
                " y: " + curYLocation +
                " \n\t px: " + prevXLocation +
                " py: " + prevYLocation +
                " \n\t ppx: " + prevPrevXLocation +
                " ppy: " + prevPrevYLocation +
                "\n\t speed: " + curMoveSpeed +
                " dir: " + curMoveDirection +
                " destX: " + xDest +
                " destY: " + yDest +
                "\n\t psect: " + prevSector +
                " csect: " + curSector +
                " c_pwr: " + curPower +
                " p_pwr: " + prevPower +
                " call: " + isCalling +
                "\n\t p_ant: " + prevAntenna +
                " c_ant: " + curAntenna +
                " vis sects: " + curVisibleSectors);
    }
    public void setName(int i) {
        name = i;
    }
    public int getName() {
        return name;
    }

    public void setIsCalling(int i) {// may need check for valid int
		if(i>=0 && i<3)
			isCalling = i;
		else
			isCalling = 2;
    }
    public int getIsCalling() {
        return isCalling;
    }
    /**
     * Based on the isCalling variable, return D, R or C for
     * Driving, Roaming or Calling respectively.
     */
    public char getIsCallingDRC() {
        char ret = '?';
        switch (isCalling) {
            case DRIVING : ret = 'D'; break;
            case ROAMING : ret = 'R'; break;
            case CALLING : ret = 'C'; break;
            default : ret = '?';
        }
        return ret;
    }

    public void setCurPower(double p) {
        curPower = p;
    }
    public double getCurPower() {
        return curPower;
    }

    public void setPrevPower(double p) {
        prevPower = p;
    }
    public double getPrevPower() {
        return prevPower;
    }

    public int getCur() {
        return cur;
    }

    public void setXLocation(int i, double x) {
        xLocation[i] = x;
    }
    public double getXLocation(int i) {
        return xLocation[i];
    }

    public void setYLocation(int i, double y) {
        yLocation[i] = y;
    }
    public double getYLocation(int i) {
        return yLocation[i];
    }

    public void setCurXLocation(double x) {
        curXLocation = x;
    }
    public double getCurXLocation() {
        return curXLocation;
    }

    public void setCurYLocation(double y) {
        curYLocation = y;
    }
    public double getCurYLocation() {
        return curYLocation;
    }

    public void setCurMoveSpeed(int i) {
        curMoveSpeed = i;
    }
    public int getCurMoveSpeed() {
        return curMoveSpeed;
    }

    public void setCurMoveDirection(double d) {
        curMoveDirection = d;
    }
    public double getCurMoveDirection() {
        return curMoveDirection;
    }

    public void setPrevXLocation(double x) {
        prevXLocation = x;
    }
    public double getPrevXLocation() {
        return prevXLocation;
    }

    public void setPrevYLocation(double y) {
        prevYLocation = y;
    }
    public double getPrevYLocation() {
        return prevYLocation;
    }

    public void setPrevPrevXLocation(double x) {
        prevPrevXLocation = x;
    }
    public double getPrevPrevXLocation() {
        return prevPrevXLocation;
    }

    public void setPrevPrevYLocation(double y) {
        prevPrevYLocation = y;
    }
    public double getPrevPrevYLocation() {
        return prevPrevYLocation;
    }

    public void setXDest(double x) {
        xDest = x;
    }
    public double getXDest() {
        return xDest;
    }

    public void setYDest(double y) {
        yDest = y;
    }
    public double getYDest() {
        return yDest;
    }

    /**
     * @param convert the integer name to convert to a string
     * @return the human readable sector name like 4C or 2B
     */
    public String getSectorString(int convert) 
	{
    	int cell;
        int sect;
        StringBuffer ret = new StringBuffer();
        if (convert != -1) 
		{   
			cell = (1 + convert / 3) + (int)'0';
			sect = (convert % 3) + (int)'A';
			ret.append((char)cell);
			ret.append((char)sect);
			return(ret.toString());
        }
		else 
		{
            return("Out");
        }
    }

	public String getCurHandoffString()
	{
		StringBuffer ret = new StringBuffer();

		if(curNumber > 1)
		{
			for(int i=0;i < curNumber-1;i++) 
				ret.append(getSectorString(curHandoff[i]) + " ");
		return (ret.toString());
		}
		else
			return(" ");
	}

	public String getPrevHandoffString()
	{
		StringBuffer ret = new StringBuffer();

		if(prevNumber > 1)
		{
			for(int i=0;i < prevNumber-1;i++) 
				ret.append(getSectorString(prevHandoff[i]) + " ");
		return (ret.toString());
		}
		else
			return(" ");
	}

	public void setPrevHandoff(int n,int s)
	{
		prevHandoff[n]=s;
	}

	public int getPrevHandoff(int n)
	{
		return prevHandoff[n];
	}

	public void setCurHandoff(int n,int s)
	{
		curHandoff[n] = s;
	}

	public int getCurHandoff(int n)
	{
		return curHandoff[n];
	}

	public void setPrevNumber(int n) // Prev Number of antennas
	{
		prevNumber = n;
	}

	public int getPrevNumber()
	{
		return prevNumber;
	}

	public void setCurNumber(int n)
	{
		curNumber = n;
	}

	public int getCurNumber()
	{
		return curNumber;
	}

    public void setPrevSector(int s) {
        prevSector = s;
    }

    public int getPrevSector() {
        return prevSector;
    }

    public String getPrevSectorString() {
        return getSectorString(prevSector);
    }

    public void setCurSector(int s) {
        curSector = s;
    }
    public int getCurSector() {
        return curSector;
    }

    public String getCurSectorString() {
        return getSectorString(curSector);
    }

    public void setPrevAntenna(int a) {
        prevAntenna = a;
    }
    public int getPrevAntenna() {
        return prevAntenna;
    }

    public String getPrevAntennaString() {
        return getSectorString(prevAntenna);
    }

    public void setCurAntenna(int a) {
        curAntenna = a;
    }
    public int getCurAntenna() {
        return curAntenna;
    }

    public String getCurAntennaString() {
        return getSectorString(curAntenna);
    }

    public void setCurVisibleSectors(String s) {
        curVisibleSectors = s;
    }

    public String getCurVisibleSectors() {
        return curVisibleSectors;
    }

    /**
     * this method converts each sector name stored in the
     * visible sector string into its integer name counterpart.
     * That is, converts 1A to 0, 4C to 11.
     *
     * @return integer array containing the sectors visible by this user.
     */
    public int [] getIntNamesForVisibleSectors() {
        int len;
        int [] ret;
    int i = 0;
        StringTokenizer tmp;
        if (curVisibleSectors == null) {
            ret = new int[1];
            ret[0] = -1;
            return ret;
        }
        tmp = new StringTokenizer(curVisibleSectors);
        len = tmp.countTokens();
        ret = new int[len];
    if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
        the.asNeededPrintln("getIntNamesForVisibleSectors");
    }
        for (i = 0; i < len; i++) {
            String t = new String(tmp.nextToken());
            int c0 = (int)t.charAt(0) - (int)'0' - 1; // will be multiplier
            int c1 = (int)t.charAt(1) - (int)'A'; // should be 0, 1 or 2
            ret[i] = (c0 * 3) + c1;
        if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                the.asNeededPrint("t: " + t);
                the.asNeededPrint("  c0: " + c0 + " c1: " + c1);
                the.asNeededPrintln("  ret[" + i + "]: " + ret[i]);
        }
        }
    if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
        the.asNeededPrintln("");
    }
    return ret;
    }

    public Boolean isMoving() {
        return (curMoveSpeed > 0 ? Boolean.TRUE : Boolean.FALSE);
    }

/* comment out since not appropriate for real modelling - used in early dev.
    public void randomInit() {
        Random randNum = new Random(); // creates random numbers with seed
                                        // based on current time
	if (DebugLevel.debugging(DebugLevel.POCAT)) {
	    setCurPower(0.0);
	} else {
            setCurPower((double)(Math.random() * 32768) % 3);
	}
//        if (getCurPower() == 0.0) {
        if (Math.random() > 0.5) { // randomly select callers
            setIsCalling(false);
        } else {
            setIsCalling(true);
        }
*/
/*
    Don't bother with x and y coords...
*/
/*
        setCurMoveSpeed((int)(Math.random() * 32768) % 152);
        setCurMoveDirection((Math.random() * 32768) % 360);
*/
/*
        the.asNeededPrintln( "randomInit " +
                            "x = " + getCurXLocation() +
                            " y = " + getCurYLocation() +
                            " P = " + getCurPower() +
                            " D = " + getCurMoveSpeed() +
                            " S = " + getCurMoveDirection());
        the.asNeededPrintln( "    maxX = " + the.map.getMaxX() +
                            "    maxY = " + the.map.getMaxY());
*/
/*
    }
*/

    /**
     * This method will initialize a user's parameters to random
     * values as appropriate for the type of model being used.
     * <p>
     * @param modl is "Arriving", "Leaving" or "Random"
     * @param cell is "7 Cell" or "4 Cell"
     * @param start is "Random" or "Perimeter"
     * @param ln is the LineSegment array which is used in setting 7 cell perimeter
     */
    public void setInitUser(String modl, String cell, String start,
                            LineSegment [] ln) {
        double modelXRadius;
        double modelYRadius;
        double x = 0.0;
        double y = 0.0;
        int i = 0, j = 0;
        Random randNum = new Random(); // creates random numbers with seed
                                        // based on current time

        // System.out.println("modl=" + modl + " cell=" + cell + " start=" + start);


		setCurPower(0.0);
		setPrevPower(0.0);
	
        double rand = Math.random();
        if (rand < 0.33) { // randomly select callers
            setIsCalling(DRIVING);
        } else if ((rand >= 0.33) && (rand < 0.67)){
            setIsCalling(ROAMING);
        } else if (rand >= 0.67) {
            setIsCalling(CALLING);
        }


        setCurMoveSpeed((int)(Math.random() * 32768) % 152);

//        setCurMoveSpeed(600);  // used to exagerate movement

        if (cell.equals("7 Cell")) {
            modelXRadius = 25.98;
            modelYRadius = 25;

            setCurXLocation((Math.random() * 32768) % modelXRadius);
            setCurYLocation((Math.random() * 32768) % modelYRadius);
            setPrevXLocation(getCurXLocation());
            setPrevYLocation(getCurYLocation());
            setPrevPrevXLocation(getCurXLocation());
            setPrevPrevYLocation(getCurYLocation());

            if (modl.equals("Arriving")) { // move towards center
                if (start.equals("Perimeter")) {
                    // Need to find all edges of this model using the
                    // Segment array.
                    LineSegment [] edgeLines = new LineSegment [ln.length];
                    for (i = 0, j = 0; i < ln.length; i++) {
                        if (ln[i].getX1() == -1) {
                            break;
                        }
                        if ((ln[i].getSectorListVal(1) == -1)) {
                            edgeLines[j] = ln[i];  // no clone - just reading
                            j++;
                        }
                    }
                    edgeLines[j] = new LineSegment(-1, -1, -1, -1, -1, -1);
                    // now j contains the number of edgeLines...
                    // now randomly pick an edge to place on
                    i = ((int)(Math.random() * 32768)) % j; // get one of the edges

                    if (edgeLines[i].isVertical()) {
                        x = edgeLines[i].getX1();
                        y = edgeLines[i].getRandomYPoint();
                    } else {
                        x = edgeLines[i].getRandomXPoint();
                        y = edgeLines[i].calcYPoint(x);
                    }
                    /*
                    if ((y = Math.random()) > 0.5) {
                        x = (Math.random() * 32768) % modelXRadius;
                        if (y > 0.75) {
                            y = 0;
                        } else {
                            y = modelYRadius;
                        }
                    } else {
                        if (y > 0.25) {
                            x = 0;
                        } else {
                            x = modelXRadius;
                        }
                        y = (Math.random() * 32768) % modelYRadius;
                    }
                    */
                    setCurXLocation(x);
                    setCurYLocation(y);
                    setPrevXLocation(getCurXLocation());
                    setPrevYLocation(getCurYLocation());
                    setPrevPrevXLocation(getCurXLocation());
                    setPrevPrevYLocation(getCurYLocation());
                    // need to set this way for this case because of
                    // model edge???
                    for (j = 0; j < xLocation.length; j++) {
                        xLocation[j] = getCurXLocation();
                        yLocation[j] = getCurYLocation();
                    }
                }
                setXDest(modelXRadius / 2);
                setYDest(modelYRadius / 2);

                setCurMoveDirection(Util.getDirection(getCurXLocation(),
                                                        getCurYLocation(),
                                                        xDest, yDest,
                                                        the.RAD_TO_DEG));
                if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                    the.asNeededPrintln("set up : " + this);
                }
            } else if (modl.equals("Leaving")) { // move away
                if (start.equals("Perimeter")) {
                setCurMoveDirection((Math.random() * 32768) % 360);
                setCurXLocation(modelXRadius/2);
                setCurYLocation(modelYRadius/2);
                setPrevXLocation(getCurXLocation());
                setPrevYLocation(getCurYLocation());
                setPrevPrevXLocation(getCurXLocation());
                setPrevPrevYLocation(getCurYLocation());
        } else { // this is a random start... need to pick random
            // then pick direction which is negative of line
            // from user to center point.
            // X and Y are already random... just set direction
            setCurMoveDirection(Util.getDirection(modelXRadius/2,
                            modelYRadius/2,
                            getCurXLocation(),
                            getCurYLocation(),
                            the.RAD_TO_DEG));
        }
            } else { // "Random"
                setCurMoveDirection((Math.random() * 32768) % 360);
            }
        } else { // 4 cell
            modelXRadius = 34.64;
            modelYRadius = 10;

            setCurXLocation((Math.random() * 32768) % modelXRadius);
            setCurYLocation(modelYRadius / 2);
            setPrevXLocation(getCurXLocation());
            setPrevYLocation(getCurYLocation());
            setPrevPrevXLocation(getCurXLocation());
            setPrevPrevYLocation(getCurYLocation());
            for (i = 0; i < xLocation.length; i++) {
                xLocation[i] = getCurXLocation();
                yLocation[i] = getCurYLocation();
            }

            if (modl.equals("Arriving")) { // left to right
                if (start.equals("Perimeter")) {
                    setCurXLocation(0);
                    setCurYLocation(modelYRadius / 2);
                    setPrevXLocation(getCurXLocation());
                    setPrevYLocation(getCurYLocation());
                    setPrevPrevXLocation(getCurXLocation());
                    setPrevPrevYLocation(getCurYLocation());
                    /*
                    for (i = 0; i < xLocation.length; i++) {
                        xLocation[i] = getCurXLocation();
                        yLocation[i] = getCurYLocation();
                    }
                    */
                }
                setCurMoveDirection(0);

                setXDest(modelXRadius);
                setYDest(modelYRadius / 2);

            } else if (modl.equals("Leaving")) { // right to left
                if (start.equals("Perimeter")) {
                    setCurXLocation(modelXRadius);
                    setCurYLocation(modelYRadius / 2);
                    setPrevXLocation(getCurXLocation());
                    setPrevYLocation(getCurYLocation());
                    setPrevPrevXLocation(getCurXLocation());
                    setPrevPrevYLocation(getCurYLocation());
                    /*
                    for (i = 0; i < xLocation.length; i++) {
                        xLocation[i] = getCurXLocation();
                        yLocation[i] = getCurYLocation();
                    }
                    */
                }
                setCurMoveDirection(180);

                setXDest(0);
                setYDest(modelYRadius / 2);

            } else { // "Random"
                x = (Math.random() * 32768) % 360;
                if (x < 180) {
                    setCurMoveDirection(0);   // Arriving left to right
                    setXDest(modelXRadius);
                    setYDest(modelYRadius / 2);
                } else {
                    setCurMoveDirection(180);  // Leaving right to left
                    setXDest(0);
                    setYDest(modelYRadius / 2);
                }
            }
        }
        if (!modl.equals("Arriving") && !start.equals("Perimeter")) {
            setInitXYLocationArray();
        }

        // Now set initial previous and calculate current sector
        int c;
        if ((c = calcCurSector()) == -1) { // then gone outside of model
            if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) {
                the.asNeededPrintln("User is outside of model (set sect to 0's)");
                the.asNeededPrintln(", x: " + getCurXLocation() +
                   ", y: " + getCurYLocation());
            }
            // Reinitialize this user
            while ((c = calcCurSector()) == -1) {
//            System.out.println("i: " + i + " x: " + x + " y: " + y);
                // and try until user is inside model
                setInitUser(modl, cell, start, ln);
            }
               // setCurSector(-1);
               // setPrevSector(-1);
               // setCurAntenna(-1);
               // setPrevAntenna(-1);
        } else {
            setCurSector(c);
            setPrevSector(c);  // set prev equal to current for initial values.
            setCurAntenna(c);
            setPrevAntenna(c);
        }
    }

    /**
     * This method will take the curMoveSpeed and curMoveDirection and
     * will figure out the hypothetical previous positions of this user.
     * This is for use by the MapArea class for calculating exact boundary
     * cross (x,y) and time of cross.
     */
    public void setInitXYLocationArray() {
        double theta;
        double reverseDir;
        double d;

        double x;
        double y;

        int i;

        int curIndex = cur;

        d = curMoveSpeed * 0.0027777;

        theta = (curMoveDirection + 180) % 360; // initialize to reverse dir
        reverseDir = theta;

        setXLocation(cur, getCurXLocation());
        setYLocation(cur, getCurYLocation());

      for(i = 0; i < 9; i++) {

        x = getXLocation(curIndex);
        y = getYLocation(curIndex);

        // decrement the circular buffer current location pointer
        // to go back in time one step (hypothetical)
        curIndex = (curIndex + 9) % 10;

        if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
            the.asNeededPrintln("inside calcPrevPositions: " + this);
        }
        if ((reverseDir >= 270) && (reverseDir < 360)) {
            // upper left
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside upper left");
            }
            theta = (360 - reverseDir) / the.RAD_TO_DEG;
            setXLocation(curIndex, x + (d * Math.cos(theta)));
            setYLocation(curIndex, y + (d * Math.sin(theta)));
        } else if ((reverseDir >= 180) && (reverseDir < 270)) {
            // upper right
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside upper right");
            }
            theta = (reverseDir - 180) / the.RAD_TO_DEG;
            setXLocation(curIndex, x - (d * Math.cos(theta)));
            setYLocation(curIndex, y + (d * Math.sin(theta)));
        } else if ((reverseDir >= 0) && (reverseDir < 90)) {
            // lower left
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside lower left");
            }
            theta = reverseDir / the.RAD_TO_DEG;
            setXLocation(curIndex, x + (d * Math.cos(theta)));
            setYLocation(curIndex, y - (d * Math.sin(theta)));
        } else if ((reverseDir >= 90) && (reverseDir < 180)) {
            // lower right
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside lower right ");
            }
               theta = (180 - reverseDir) / the.RAD_TO_DEG;
            setXLocation(curIndex, x - (d * Math.cos(theta)));
            setYLocation(curIndex, y - (d * Math.sin(theta)));
        } else {
            System.err.println("Incorrect moveDirection! " + reverseDir);
        }
      }

//        for (i = 0; i < xLocation.length; i++) {
//            the.asNeededPrintln("i:" + i + " x:" + getXLocation(i) + " y:" + getYLocation(i));
//        }
        if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
            the.asNeededPrintln("exitting calcNextPosition: " + this);
        }
    }

    /**
     * This method will figure out the next position for this user
     * based on its attributes and the current kind of model - Model
     * information is passed in via the parameters to this function.
     * <p>
     * If necessary, the user is regenerated.
     *
     * @param modl is "Arriving", "Leaving" or "Random"
     * @param cell is "7 Cell" or "4 Cell"
     * @param behavior is "Constant" or "Regenerate"
     * @param start is "Random" or "Perimeter"
     * @see User#setNextStats
     */
    public void setNextStats(String modl, String cell, String behavior,
                            String start) {
        // move user in current direction by one increment (10 seconds?)
        // 10 seconds = 0.0027777 hours.  so move 0.27777km at 100km/hour

        // Future... should have an object oriented method of getting the
        // next Stats without this big if statement.  Same for initialize above

        // for now, don't change speed or direction
        double theta = curMoveDirection / the.RAD_TO_DEG; // convert to radians

        double modelXRadius;
        double modelYRadius;
        double x;
        double y;

    int c;
        int speed;
        Random randNum = new Random(); // creates random numbers with seed
                                        // based on current time
        speed = getCurMoveSpeed();

        // Same calculations can be used for 4 cell or 7 cell patterns!!
        calcNextPosition(modl, cell);

        // Now set previous and calculate current sector
        setPrevSector(getCurSector());
        setPrevAntenna(getCurAntenna());
		setPrevPower(getCurPower());
/*
 * NOTE... Current and Previous Sectors may not be correct or worth anything
 * the first time thru!!!
 */
        if ((c = calcCurSector()) == -1) 
		{ // then gone outside of model
            if (DebugLevel.debugging(DebugLevel.USER_DEBUGGING)) 
			{
                the.asNeededPrint("User is outside of model");
                the.asNeededPrintln(", x: " + getCurXLocation() +
                   ", y: " + getCurYLocation());
			}
            if (behavior.equals("Regenerate")) 
			{
                // Reinitialize this user
				while ((c = calcCurSector()) == -1) 
				{
                // and try until user is inside model
                setInitUser(modl, cell, start, the.model.getLineSeg());
				}
			} 
			else 
			{
            // if "Constant", then just let user continue on
            // its merry way
                setCurSector(-1);
                setPrevSector(-1);
                setCurAntenna(-1);
                setPrevAntenna(-1);
            }
        } 
		else if (cell.equals("7 Cell") && modl.equals("Arriving")) 
		{
			if (Util.isEquiv(getCurXLocation(), getXDest()) &&
			    Util.isEquiv(getCurYLocation(), getYDest())) 
			{
                if (behavior.equals("Regenerate")) 
				{
                // Reinitialize this user
					setInitUser(modl, cell, start,
						the.model.getLineSeg());
					while ((c = calcCurSector()) == -1) 
					{
                    // and try until user is inside model
                    setInitUser(modl, cell, start,
                    the.model.getLineSeg());
					}
				}
			}
        }
        setCurSector(c);
        setCurAntenna(c);
    }

    /**
     * This method will call the map's calcCurSector to calculate the
     * current sector based on this users (x,y) location.
     *
     * @return the integer sector name where this user is located.
     * This will be -1 if outside the model.
     */
    public int calcCurSector() {
        // use map to calcultate.... since map has coordinate system stuff
        return(the.map.calcCurSector(getCurXLocation(), getCurYLocation()));
    }

    public double getXBoundaryCross() {
        return xBoundaryCross;
    }
    public void setXBoundaryCross(double x) {
        xBoundaryCross = x;
    }

    public double getYBoundaryCross() {
        return yBoundaryCross;
    }
    public void setYBoundaryCross(double y) {
        yBoundaryCross = y;
    }

    /**
     * This method will figure out the next position for this user
     * based on its attributes and the current kind of model - Model
     * information is passed in via the parameters to this function.
     * <p>
     * This method does all the work needed for the setNextStats method.
     * <p>
     * Calculations for next position use cos and sin of an angle to
     * determine the distance traveled.  Currently, distance travelled
     * in one time increment is hard coded for 10 second increments:
     * 0.0027777 * the curMoveSpeed will equal the distance travelled (d).
     * For example: x + (d * Math.cos(curMoveDirection)) is the new value of x.
     * <p>
     * @param modl is "Arriving", "Leaving" or "Random"
     * @param cell is "7 Cell" or "4 Cell"
     * @see User#setNextStats
     */
    public void calcNextPosition(String modl, String cell) {
        double theta;
        double d;

        double x;
        double y;

        int i;

        x = curXLocation;
        y = curYLocation;

        // increment the circular buffer current location pointer
        cur = (cur + 1) % 10;

        d = curMoveSpeed * 0.0027777;

        theta = curMoveDirection; // initialize

        if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
            the.asNeededPrintln("inside calcNextPosition: " + this);
        }
        if ((curMoveDirection >= 270) && (curMoveDirection < 360)) {
            // upper left
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside upper left");
            }
            theta = (360 - curMoveDirection) / the.RAD_TO_DEG;
            setCurXLocation(x + (d * Math.cos(theta)));
            setCurYLocation(y + (d * Math.sin(theta)));
            setXLocation(cur, getCurXLocation());
            setYLocation(cur, getCurYLocation());
        } else if ((curMoveDirection >= 180) && (curMoveDirection < 270)) {
            // upper right
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside upper right");
            }
            theta = (curMoveDirection - 180) / the.RAD_TO_DEG;
            setCurXLocation(x - (d * Math.cos(theta)));
            setCurYLocation(y + (d * Math.sin(theta)));
            setXLocation(cur, getCurXLocation());
            setYLocation(cur, getCurYLocation());
        } else if ((curMoveDirection >= 0) && (curMoveDirection < 90)) {
            // lower left
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside lower left");
            }
            theta = curMoveDirection / the.RAD_TO_DEG;
            setCurXLocation(x + (d * Math.cos(theta)));
            setCurYLocation(y - (d * Math.sin(theta)));
            setXLocation(cur, getCurXLocation());
            setYLocation(cur, getCurYLocation());
        } else if ((curMoveDirection >= 90) && (curMoveDirection < 180)) {
            // lower right
            if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                the.asNeededPrintln("inside lower right ");
            }
               theta = (180 - curMoveDirection) / the.RAD_TO_DEG;
            setCurXLocation(x - (d * Math.cos(theta)));
            setCurYLocation(y - (d * Math.sin(theta)));
            setXLocation(cur, getCurXLocation());
            setYLocation(cur, getCurYLocation());
        } else {
            System.err.println("Incorrect moveDirection! " + curMoveDirection);
        }
        setPrevPrevXLocation(getPrevXLocation());
        setPrevPrevYLocation(getPrevYLocation());
        setPrevXLocation(x); // x and y have the old values of CurX and CurY
        setPrevYLocation(y);

//        for (i = 0; i < xLocation.length; i++) {
//            the.asNeededPrintln("i:" + i + " x:" + getXLocation(i) + " y:" + getYLocation(i));
//        }

        // check to see if user has passed the DestX,DestY... if so, set
        // speed to 0 and curX/Y to Dest, etc.
        if (modl.equals("Arriving") && cell.equals("7 Cell")) {
          if (getCurMoveSpeed() != 0) {
            double a, b, c;
            a = Util.calcLength(getPrevXLocation(), getPrevYLocation(),
                                getCurXLocation(), getCurYLocation());
            b = Util.calcLength(getPrevXLocation(), getPrevYLocation(),
                                getXDest(), getYDest());
            c = Util.calcLength(getCurXLocation(), getCurYLocation(),
                                getXDest(), getYDest());

            if (Util.isEquiv(a, (b + c))) { // this means at the center!!!
                if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
                    the.asNeededPrintln("Got straight line calc to work");
                }
                setCurMoveSpeed(0);
                setCurXLocation(getXDest());
                setCurYLocation(getYDest());
            }
/*
             else {// with corrected floating point ==, should not need
                theta = Util.calcTheta(getPrevXLocation(), getPrevYLocation(),
                                    getCurXLocation(), getCurYLocation(),
                                    getXDest(), getYDest());
                theta = theta * the.RAD_TO_DEG;

                // This check is done to see if there was error in the degrees
                // to radians calculations...
                if (theta < 5) { // pretty close to zero degrees...
                    System.out.println("Should not be here.... ");
                    System.out.println("Straight line calc did not work");
                    setCurMoveSpeed(0);
                    setCurXLocation(getXDest());
                    setCurYLocation(getYDest());
                }
            }
*/
          }
        }
        if (DebugLevel.debugging(DebugLevel.MOVE_DEBUGGING)) {
            the.asNeededPrintln("exitting calcNextPosition: " + this);
        }
    }

	// Set flag for visible sector "sector", 1=vissible 0=not
	public void SetVisibleSector(int sector,int i)
	{
		VisibleSectors[sector]=i;
	}

	public int IsVisible(int sector)
	{
		return(VisibleSectors[sector]);
	}

	public void calcPathloss(int fadingmodel)
	{
		for(int i=0;i < the.sector.length;i+=3)
		{
			if(fadingmodel==0)
			{	//distance in meters
				double d = getDist(curXLocation,curYLocation
							,the.sector[i].getBaseX(),the.sector[i].getBaseY());
				if(d > 5)
					Pathloss[i/3] = Math.pow(d,-4);
				else
					Pathloss[i/3] = 0.0016; /* minimum distance = 5 m 5^-4*/
			}
		}
	}

	public double getPathloss(int s)
	{
		return (Pathloss[s/3]);
	}

	private double getDist(double x,double y,double bx,double by)
	{
		double dx = x-bx;
		double dy = y-by;
		//returns distance in meters
		return (1000 * Math.sqrt(dx*dx + dy*dy));
	}

	public int IsPossibleAnt(int antenna)
	{
		return (possibleAnt[antenna]);
	}

	public void calcPossibleAnt(int type,int max_dist)
	{
		int bs = the.model.getCells(); 
		if(type==0) /*Use Maxdistance*/
		{
			for(int i=0; i < bs;i++)
			{
				possibleAnt[i] = 0;
				if(getDist(curXLocation,curYLocation,
						the.sector[i*3].getBaseX(),
						the.sector[i*3].getBaseY()) < max_dist)
				{
					possibleAnt[i] = 1;
				}
			}
		}
		else
		{
			double [] dist  = new double [8]; 
			int [] closest = new int [7];
			dist[7] = 1e10;
			dist[0] = getDist(curXLocation,curYLocation,
						the.sector[0].getBaseX(),
						the.sector[0].getBaseY());
			closest[0]=0;
			for(int i=1;i<type;i++)
			{
				closest[i]=7;
			}
			for(int i=1;i<bs;i++)
			{
				possibleAnt[i] = 0;
				dist[i] = getDist(curXLocation,curYLocation,
						the.sector[i*3].getBaseX(),
						the.sector[i*3].getBaseY());
				for(int j=0;j<type;j++)
				{
					if(dist[i] < dist[closest[j]])
					{
						for(int k=type-1;k>j;k--)
						{
							closest[k]=closest[k-1];
						}
					closest[j]=i;
					j=type;
					}
				}
			}
			for(int i=0;i<type;i++)
			{
				possibleAnt[closest[i]] = 1;
			}
		}
	}
}
