/*
 * UserStats Panel
 *
 * Author: M. Heidi McClure
 * Date Started: 9/1/96
 * Date Last Revised: 4/12/97
 * Revised By Jonas Hedlind 
 * Date: June 97
 */

import java.awt.*;
import java.io.*; // for output stream
import java.util.*; // for Date

/**
 * This class is the bottom portion of UTMOST2.  It contains the User
 * Statistics for the a selected user and contains the main controls
 * for executing model steps.
 * <p>
 * This class also controls the creation, opening, writing to and closing
 * of files used to record results of the model runs.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see MapPanel
 * @see UTMOST2
 * @see ModelSetup
 * @see MapArea
 */
public class UserStats extends Panel {
    private EtchBox westSide;
    private Label uSpeed = new Label("Speed:");
    private Label uSpeedText = new Label("");
    private Label uDirection = new Label("Move Dir:");
    private Label uDirectionText = new Label("");

    private Label uCurLoc = new Label("Cur Loc:");
    private Label uCurLocText = new Label("");
    private Label uPrevLoc = new Label("Prev Loc:");
    private Label uPrevLocText = new Label("");

    private Label uCalling = new Label("Calling?");
    private Label uCallingText = new Label("");

    private Label uCurPower = new Label("Cur Pwr:");
    private Label uCurPowerText = new Label("");
    private Label uPrevPower = new Label("Prev Pwr:");
    private Label uPrevPowerText = new Label("");

    private Label uCurAntenna = new Label("Cur Ant:");
    private Label uCurAntennaText = new Label("");
    private Label uPrevAntenna = new Label("Prev Ant:");
    private Label uPrevAntennaText = new Label("");

	private Label prevhandoff = new Label("Prev Handoff:");
	private Label prevhandoffants = new Label("                 ");
	private Label curhandoff = new Label("Cur Handoff:");
	private Label curhandoffants = new Label("                 ");

    private Button uGoButton = new Button("->>");
	private Button GoBackButton = new Button("<<-");
	private Button GoAgainButton = new Button(" GO ");
	//private Label GoButtonText = new Label("Go   ");

    private Label uRun = new Label("Run");
    private TextField uNumSteps = new TextField("1", 2);
    private Label uSteps = new Label("steps");
    private Label uTotal = new Label("0 steps so far");

    private int steps;
    private int curStep; //current number of steps simulated
	private int curStepDisplayed; //current displayed step
    private boolean firstGo;

    private UTMOST2 the;
	private POCAT PowerControl;
	

    PrintStream oactivity;
    PrintStream oboundary;
    DataInputStream iPocat;
	DataInputStream iActivity;
    DataInputStream iRacesim;
    PrintStream odone;
    PrintStream osectors;
	PrintStream odataFile;
	DataInputStream idataFile;

    StringBuffer activityFilename = new StringBuffer();
    StringBuffer boundaryFilename = new StringBuffer();
    StringBuffer pocatFilename = new StringBuffer();
    StringBuffer racesimFilename = new StringBuffer();
    StringBuffer doneFilename = new StringBuffer();
    StringBuffer pocatDoneFilename = new StringBuffer();
    StringBuffer racesimDoneFilename = new StringBuffer();
	String		 modelDataFilename;
    String       sectorDataFilename;
    String date;

    boolean [] crossedLines; // flag for lines which have already been
                            // crossed.

    /**
     * Constructor which will create a UserStats object.
     *
     * @param u is the UTMOST2 object which called this constructor
     */
    public UserStats(UTMOST2 u) {
        the = u;
        steps = 1;
        curStep = 0;
		curStepDisplayed = 0;
        firstGo = true;
// West Side Stuff
        Panel west = new Panel();
        west.setLayout(new GridLayout(3,6,10,3));
        west.add(uSpeed);
        west.add(uSpeedText);
        west.add(uDirection);
        west.add(uDirectionText);
        west.add(uCalling);
        west.add(uCallingText);

        west.add(uPrevLoc);
        west.add(uPrevLocText);
        west.add(uPrevPower);
        west.add(uPrevPowerText);
        west.add(uPrevAntenna);
        west.add(uPrevAntennaText);

        west.add(uCurLoc);
        west.add(uCurLocText);
        west.add(uCurPower);
        west.add(uCurPowerText);
        west.add(uCurAntenna);
        west.add(uCurAntennaText);
		
		Panel swest = new Panel();
        swest.setLayout(new GridLayout(1,4,10,3));
		swest.add(prevhandoff);
		swest.add(prevhandoffants);
		swest.add(curhandoff);
		swest.add(curhandoffants);
		
		Panel ustats = new Panel();
		ustats.setLayout(new BorderLayout(2,2));
		ustats.add("North",west);
		ustats.add("South",swest);
		
        // use lots of spaces to have enough room when user name is inserted
        westSide = new EtchBox(ustats,
            "          Selected User Status             ");

// East Side Stuff
        Panel east = new Panel();
        east.setLayout(new BorderLayout(10,1));
        Panel eastNorth = new Panel();
		eastNorth.add(GoBackButton);
		//eastNorth.add(GoButtonText);
		eastNorth.add(GoAgainButton);
		eastNorth.add(uGoButton);
		
		
		east.add("North", eastNorth);

        Panel eastCenter = new Panel();
        eastCenter.add(uRun);
        eastCenter.add(uNumSteps);
        eastCenter.add(uSteps);

        east.add("Center", eastCenter);

        uTotal.setText("" + curStep + " steps so far");

        east.add("South", uTotal);


        EtchBox eastBox = new EtchBox(east, "Controls");
        //setLayout(new BorderLayout());

        add(westSide);
        add(eastBox);

        // this is potentially a bug when other cell patterns are
        // introduced - right now 7 cell is biggest and is the
        // default, so at the beginning, the lenght of lineSeg is
        // biggest
        crossedLines = new boolean [(the.model.getLineSeg()).length];

        for (int j = 0; j < crossedLines.length; j++) {
            crossedLines[j] = false;
        }

    }

    public void setUSpeedText(String s) {
        uSpeedText.setText(s);
    }

    public void setUDirectionText(String s) {
        uDirectionText.setText(s);
    }

    public void setUCurLocText(String s) {
        uCurLocText.setText(s);
    }

    public void setUPrevLocText(String s) {
        uPrevLocText.setText(s);
    }

    public void setUCallingText(String s) {
        uCallingText.setText(s);
    }

    public void setUCurPowerText(String s) {
        uCurPowerText.setText(s);
    }

    public void setUPrevPowerText(String s) {
        uPrevPowerText.setText(s);
    }

    public void setUCurAntennaText(String s) {
        uCurAntennaText.setText(s);
    }

    public void setUPrevAntennaText(String s) {
        uPrevAntennaText.setText(s);
    }

	public void setCurHandoffText(String s){
		curhandoffants.setText(s);
	}

	public void setPrevHandoffText(String s){
		prevhandoffants.setText(s);
	}


    public void setCurStep(int i) {
        curStep = i;
    }

    public void setFirstGo(boolean b) {
        firstGo = b;
    }

    public void closeOStreams() {
        oactivity.close();
		if(the.generateBoundary)
			oboundary.close();
    }

    /**
     * This method opens the activity and boundary files for this
     * model run.
     */
    public void openOStreams() {
        try {
            oactivity = new PrintStream(
                new FileOutputStream(activityFilename.toString()));
        } catch (Exception e) {
            the.asNeededPrintln("problem when opening " +
		activityFilename.toString());
        }
		if(the.generateBoundary){
			try {
				oboundary = new PrintStream(
			    new FileOutputStream(boundaryFilename.toString()));
			} catch (Exception e) {
				the.asNeededPrintln("problem when opening " +
				boundaryFilename.toString());
			}
		}
    }

	public void openActivityStream() {
        try {
            iActivity = new DataInputStream(
                new FileInputStream(activityFilename.toString()));
        } catch (Exception e) {
            the.asNeededPrintln("problem when opening " +
		    activityFilename.toString());
        }
    }

    /**
     * This method opens the POCAT input files for this
     * model run.
     */
    public void openPocatStream() {
        try {
            iPocat = new DataInputStream(
                new FileInputStream(pocatFilename.toString()));
        } catch (Exception e) {
            the.asNeededPrintln("problem when opening " +
		    pocatFilename.toString());
        }
    }

    /**
     * This method opens the RACESIM input files for this
     * model run.
     */
    public void openRacesimStream() {
        try {
            iRacesim = new DataInputStream(
                new FileInputStream(racesimFilename.toString()));
        } catch (Exception e) {
            the.asNeededPrintln("problem when opening " +
		    racesimFilename.toString());
        }
    }

    /**
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
        if (evt.target == uGoButton) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Go button pressed");
            }
            goButtonAction();
        }
        if (evt.target == uNumSteps) {
            if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                the.asNeededPrintln("Num of Steps Changed (and ENTER pressed)");
            }
            goButtonAction();
        }
		if (evt.target == GoBackButton)
		{
			goBackButtonAction();
		}
		if (evt.target == GoAgainButton)
		{
			goAgainButtonAction();
		}
        return super.action(evt, arg);
    }

    /**
     * This method runs as many steps of the model as indicated in the
     * number of steps to run field.  Running of a step includes updating
     * each user's position, calculating boundary crossing if necessary,
     * and any other stuff needed.
     * <p>
     * This is where Discrete Event Simulator would be hooked in!!!
     *
     */
    public void goButtonAction() {
    	// set cursor to WAIT_CURSOR
		if(curStep == curStepDisplayed)
		{
    	the.setCursor(Frame.WAIT_CURSOR);

        // in case users haven't been initialized...
            if (Util.isEquiv(the.users[0].getCurMoveDirection(), -1)) {
                the.asNeededPrintln(
                "First initialize model by selecting" +
                    " items in the Model Setup Area of GUI");
                return;
            }
            // in case steps changed and <enter> not pressed
            steps = new Integer(uNumSteps.getText()).intValue();

//            if (curStep == 0) { // first time through, so create new dir
            if (firstGo) { // first time through, so create new dir
                date = getDateString();
                activityFilename.setLength(0);
                activityFilename.append("utmost_files" + the.FILE_SEP +
                                        "activity" + the.FILE_SEP);
                activityFilename.append(date);
                
				
				boundaryFilename.setLength(0);
				boundaryFilename.append("utmost_files" + the.FILE_SEP +
                                       "boundary" + the.FILE_SEP);
				boundaryFilename.append(date);
				
                sectorDataFilename = new String(activityFilename.toString() +
                                                the.FILE_SEP + "sector_data");
				modelDataFilename = new String(activityFilename.toString() +
                                                the.FILE_SEP + "model_data");

                if (!(new File(activityFilename.toString()).exists())) {
                    new File(activityFilename.toString()).mkdirs();
                    new File(boundaryFilename.toString()).mkdirs();
                    if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                        the.asNeededPrintln("dir created is: " +
                                activityFilename.toString());
                        the.asNeededPrintln("dir created is: " +
                                boundaryFilename.toString());
                    }
                }
                writeSectorDataFile();
				
                the.mapPanel.setDirText("Results in " + date);
				//Jonas
				if(the.PowerControlType == 1) 
					PowerControl = new POCAT(the,the.PowerControlSetting,date);
            }
            if (the.PowerControlType>1  && firstGo) 
			{ //push twice to start
                firstGo = false;
    	        // set cursor back to original cursor type
    	        the.setCursor(Frame.DEFAULT_CURSOR);
                return;
            } 
			else 
			{
              firstGo = false;
              for (int j = 0; j < steps; j ++) 
			  {
                curStep++;
				curStepDisplayed=curStep;
                if (DebugLevel.debugging(DebugLevel.TIMING_DEBUGGING)) 
				{
                    DebugLevel.printTime(" when step " + curStep +
                        " started");
                }
                activityFilename.setLength(0);
                doneFilename.setLength(0);
                activityFilename.append("utmost_files" + the.FILE_SEP +
                                        "activity" + the.FILE_SEP);
                activityFilename.append(date);
                activityFilename.append(the.FILE_SEP + curStep);
                doneFilename.append(activityFilename + "done");
				
				if(the.generateBoundary){
					boundaryFilename.setLength(0);
					boundaryFilename.append("utmost_files" + the.FILE_SEP +
                                        "boundary" + the.FILE_SEP);
					boundaryFilename.append(date);
					boundaryFilename.append(the.FILE_SEP + curStep);
				}

                if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) 
				{
                    the.asNeededPrintln("activityFilename is: " +
                                activityFilename.toString());
                    the.asNeededPrintln("boundaryFilename is: " +
                                boundaryFilename.toString());
                }
                openOStreams();
                if (DebugLevel.debugging(DebugLevel.SIMULATOR)) 
				{
                    if (DebugLevel.debugging(DebugLevel.POCAT)) 
					{
                        pocatFilename.setLength(0);
                        pocatDoneFilename.setLength(0);
                        pocatFilename.append("other_sim_files" + the.FILE_SEP +
                                        "chan_assign" + the.FILE_SEP);
                        if (the.SIM_DIR_NAME.length() == 0) 
						{
                            //pocatFilename.append("970314115541");
                            pocatFilename.append(date);
                        } 
						else 
						{
                            pocatFilename.append(the.SIM_DIR_NAME);
                        }
                        pocatFilename.append(the.FILE_SEP + curStep);
                        pocatDoneFilename.append(pocatFilename +
                                        "done");
                    } 
					else if (DebugLevel.debugging(DebugLevel.RACESIM)) 
					{
                        racesimFilename.setLength(0);
                        racesimDoneFilename.setLength(0);
                        racesimFilename.append("other_sim_files" + the.FILE_SEP +
                                        "racesim" + the.FILE_SEP);
                        if (the.SIM_DIR_NAME.length() == 0) 
						{
                            //racesimFilename.append("970314115541");
                            racesimFilename.append(date);
                        } 
						else 
						{
                            racesimFilename.append(the.SIM_DIR_NAME);
                        }
                        racesimFilename.append(the.FILE_SEP + curStep);
                        racesimDoneFilename.append(racesimFilename +
                                        "done");
                    }
                }
                for (int i = 0; i < the.map.getNumUsers(); i++) {
                    if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                        the.asNeededPrintln(
                    "setNextStats for user " + i);
                    }
            // currently setNextStats is hard coded to move in
            // 10 second increments....
                    the.users[i].setNextStats(the.model.getModel(),
                        the.model.getPattern(), the.model.getBehavior(),
                        the.model.getStartType());
                    writeResults(i);
                }
				
				//POCAT
				if(the.PowerControlType == 1)
					PowerControl.calcNextStep(curStep,PowerControl.Go);

                closeOStreams();

                // create and close the done file for this step
                try {
                    odone = new PrintStream(
                        new FileOutputStream(doneFilename.toString()));
                } catch (Exception e) {
                    the.asNeededPrintln("problem when opening " +
        		    doneFilename.toString());
                }
                odone.close();

                if (the.PowerControlType == 2)//DebugLevel.debugging(DebugLevel.SIMULATOR)) 
				{//Wait for #done file
                        waitForPocatDone();
                        readPocatFile(); 
				}
				else if (the.PowerControlType == 3)//DebugLevel.debugging(DebugLevel.RACESIM)) 
					{//RACESIM
                        waitForRacesimDone();
                        readRacesimFile();
                    }
                //}

                uTotal.setText("Step "+ curStepDisplayed +" of " + curStep );
				writeModelDataFile();
                the.map.update(the.map.getGraphics()); // this forces redraw after each
                                                        // step...
                if (DebugLevel.debugging(DebugLevel.TIMING_DEBUGGING)) {
                    DebugLevel.printTime(" when step " + curStep +
                        " completed");
                }
              }
            }
		}
		else
		{
			curStepDisplayed++;
			activityFilename.setLength(0);
            doneFilename.setLength(0);
            activityFilename.append("utmost_files" + the.FILE_SEP +
                                        "activity" + the.FILE_SEP);
            activityFilename.append(date);
            activityFilename.append(the.FILE_SEP + curStepDisplayed);
            doneFilename.append(activityFilename + "done");
			pocatFilename.setLength(0);
            pocatDoneFilename.setLength(0);
            pocatFilename.append("other_sim_files" + the.FILE_SEP +
                                     "chan_assign" + the.FILE_SEP);
            pocatFilename.append(date);
            pocatFilename.append(the.FILE_SEP + curStepDisplayed);
            pocatDoneFilename.append(pocatFilename + "done");

			waitForPocatDone();
            readPocatFile();
			readActivityFile();
			uTotal.setText("Step "+ curStepDisplayed +" of " + curStep );
			the.map.update(the.map.getGraphics());
		}
    	// set cursor back to original cursor type
    	the.setCursor(Frame.DEFAULT_CURSOR);

    }

	private void goBackButtonAction()
	{
		the.asNeededPrintln("Go Back Button pressed");
		if (curStepDisplayed > 1)
		{
			the.setCursor(Frame.WAIT_CURSOR);
			curStepDisplayed--;
			activityFilename.setLength(0);
            doneFilename.setLength(0);
            activityFilename.append("utmost_files" + the.FILE_SEP +
                                        "activity" + the.FILE_SEP);
            activityFilename.append(date);
            activityFilename.append(the.FILE_SEP + curStepDisplayed);
            doneFilename.append(activityFilename + "done");
			pocatFilename.setLength(0);
            pocatDoneFilename.setLength(0);
            pocatFilename.append("other_sim_files" + the.FILE_SEP +
                                     "chan_assign" + the.FILE_SEP);
            pocatFilename.append(date);
            pocatFilename.append(the.FILE_SEP + curStepDisplayed);
            pocatDoneFilename.append(pocatFilename + "done");

			waitForPocatDone();
            readPocatFile();
			readActivityFile();
			uTotal.setText("Step "+ curStepDisplayed +" of " + curStep );
			the.map.update(the.map.getGraphics());
		}
	the.setCursor(Frame.DEFAULT_CURSOR);
	}

	private void goAgainButtonAction(){
			// set cursor to WAIT_CURSOR
		if(the.PowerControlType == 1 && !firstGo){
    		the.setCursor(Frame.WAIT_CURSOR);
			for (int i = 0; i < the.map.getNumUsers(); i++) {
			the.users[i].setCurVisibleSectors(calcVisibleSectorsOptimum(
                    the.users[i].getCurXLocation(), the.users[i].getCurYLocation(),
                    the.sector,i));
			}
        	PowerControl.calcNextStep(curStep,PowerControl.GoAgain);

            the.map.update(the.map.getGraphics()); // this forces redraw after each
		}  
		the.setCursor(Frame.DEFAULT_CURSOR);
	}

    /**
     * This method records the current sector map into the activity
     * file directory.
     */
    private void writeSectorDataFile() {
        StringBuffer line = new StringBuffer();

        try {
            osectors = new PrintStream(
                new FileOutputStream(sectorDataFilename));
        } catch (Exception e) {
            the.asNeededPrintln("problem when opening " + sectorDataFilename);
        }
        int corner;
        for (int i = 0; i < the.sector.length; i++) {
            corner = the.sector[i].getCornerName();
            if (corner == -1) { // beyond the bounds of this set of sectors...
                break;
            }
            line.setLength(0);
            line.append(new Format("%4d").form(i));
            line.append(" ");
            switch (corner) {
                case 1 :
                    line.append(new Format("%3d").form(150));
                    line.append(" ");
                    break;
                case 2 :
                    line.append(new Format("%3d").form(30));
                    line.append(" ");
                    break;
                case 3 :
                    line.append(new Format("%3d").form(270));
                    line.append(" ");
                    break;
                default :
                    the.asNeededPrintln("Incorrect CornerName found: " +
                                        the.sector[i].getCornerName());
            }
            line.append(new Format("%7.2f").form(the.sector[i].getBaseX()));
            line.append(" ");
            line.append(new Format("%7.2f").form(the.sector[i].getBaseY()));
            line.append(" ");
            line.append(new Format("%7.2f").form(
                                    the.sector[i].getRange() / the.RADIUS));
            line.append(" ");

            osectors.println(line.toString());
        }
        osectors.close();
    }

    /**
     * This method writes the activity results out to the appropriate
     * file.  If necessary, it will also call the writeBoundaryResults()
     * method.
     *
     * @param i this is the index into the user array for this user.
     */
    private void writeResults(int i) {
        StringBuffer line = new StringBuffer();

        char crossed;
        crossed = 'N';
        if (the.users[i].getCurSector() != the.users[i].getPrevSector()) {
            if (  !(Util.isEquiv(the.users[i].getCurXLocation(),
                    the.users[i].getXDest()) &&
                    Util.isEquiv(the.users[i].getCurYLocation(),
                    the.users[i].getYDest()))) {
                crossed = 'Y';

				if(the.generateBoundary){				
					if (DebugLevel.debugging(DebugLevel.NO_ADJ_OPT_DEBUGGING)) {
						writeBoundaryResults(i);
					} else {
						writeBoundaryResultsOptimum(i);
					}	
					for (int j = 0; j < crossedLines.length; j++) { // reinitialize
						crossedLines[j] = false;
					}
				}
            }
        }
        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            the.asNeededPrintln(
            "curSectorString: " + the.users[i].getCurSectorString());
        }
        if (DebugLevel.debugging(DebugLevel.NO_VIS_OPT_DEBUGGING)) {
            the.users[i].setCurVisibleSectors(calcVisibleSectors(
                    the.users[i].getCurXLocation(), the.users[i].getCurYLocation(),
                    the.sector));
        } else {
            the.users[i].setCurVisibleSectors(calcVisibleSectorsOptimum(
                    the.users[i].getCurXLocation(), the.users[i].getCurYLocation(),
                    the.sector,i));
        }
		//\
        if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
            the.asNeededPrintln(
            "curVisibleSectors: " + the.users[i].getCurVisibleSectors());
        }

    line.append(new Format("%4d").form(curStep));
    line.append(" ");
    line.append(new Format("%4d").form(i));
    line.append(" ");
    line.append(new Format("%6.2f").form(the.users[i].getCurXLocation()));
    line.append(" ");
    line.append(new Format("%6.2f").form(the.users[i].getCurYLocation()));
    line.append(" ");
    line.append(new Format("%3d").form(the.users[i].getCurMoveSpeed()));
    line.append(" ");
    line.append(new Format("%8.2f").form(the.users[i].getCurMoveDirection()));
    line.append(" ");
            // currently user.setNextStats is hard coded for 10
            // second increments in the model
    line.append(new Format("%10.2f").form((double)(curStep * the.TIME_INCREMENT)));
    line.append(" ");
    line.append(new Format("%4.2f").form(the.users[i].getCurPower()));
    line.append(" ");
    line.append(new Format("%c").form(the.users[i].getIsCallingDRC()));
    line.append(" ");
    line.append(new Format("%3s").form(the.users[i].getCurAntennaString()));
    line.append(" ");
    line.append(new Format("%3s").form(the.users[i].getCurSectorString()));
    line.append(" ");
    line.append(new Format("%c").form(crossed));
    line.append(" ");
    line.append(new Format("%s").form(the.users[i].getCurVisibleSectors()));

    oactivity.println(line.toString());
    }

    /**
     * Visible sectors are calculated using the law of cosines:
     *
     * for example:
     *                  C
     *                  /\(baseX,baseY)
     *                /    \
     *             a/        \b
     *            /            \
     *     (x,y)/B______________A\(segX,segY) where segX and Y are part of sector
     *                  c
     *
     *  a, b and c are calculated by Util.calcLength() method which is a simple
     *                       2          2
     *  length sqroot((x2-x1)  + (y2-y1) )
     *
     *                         2    2    2
     *                   -1 / a  + b  - c  \
     *  the angle C = cos   |--------------|
     *                      \     2ab      /
     *
     * Then using the fact that if (x,y) is visible by a sector, then the
     * two triangles created with (x,y), (baseX, baseY) and the two line
     * segments for the sector that include (baseX, baseY) will both have an
     * angle C at point (baseX, baseY) which is less than 120 degrees.
     */
    private String calcVisibleSectors(double x, double y, Sector [] s) {
        StringBuffer ret = new StringBuffer();;
        LineSegment [] ln = the.model.getLineSeg();
        double C0, C1, C2;
        double bx, by;
        LineSegment [] seg = new LineSegment [3];

        double DEG120 = 120 / the.RAD_TO_DEG;

//        System.out.println("120 degrees = " + DEG120);

        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            the.asNeededPrintln(
            "inside calcVisibleSectors. x = " + x + " y = " + y);
        }

        for (int i = 0; i < s.length; i = i + 3) { // for each sector...
            if (s[i].getName() == -1) {
                break;
            }
            bx = s[i].getBaseX();
            by = s[i].getBaseY();
            seg = findSegs(ln, bx, by); // this will give the three segments
                                        // which include (bx,by)
            if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                the.asNeededPrintln("seg[0]: " + seg[0] + "\nseg[1]: " +
                    seg[1] +"\nseg[2]: " + seg[2]);
            }
            C0 = Util.calcTheta(x, y, bx, by, seg[0]);
            C1 = Util.calcTheta(x, y, bx, by, seg[1]);
            C2 = Util.calcTheta(x, y, bx, by, seg[2]);
            if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                the.asNeededPrintln(
                "     C0: " + C0 + " C1: " + C1 + " C2: " + C2);
                the.asNeededPrintln("     x: " + x + " y: " + y +
                " bx: " + bx + " by: " + by);
                the.asNeededPrintln("     seg[0]: " + seg[0]);
                the.asNeededPrintln("     seg[1]: " + seg[1]);
                the.asNeededPrintln("     seg[2]: " + seg[2]);
            }

            if ((C0 <= DEG120) && (C1 <= DEG120)) {
                if ((seg[0].getSectorListVal(0) ==
                                                seg[1].getSectorListVal(0)) ||
                    (seg[0].getSectorListVal(0) ==
                                                seg[1].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[0].getSectorListVal(0) +
                                        " is visible to this point" +
                                        " (seg[0].getSectorListVal(0))");
                    }
                    ret.append(s[seg[0].getSectorListVal(0)].getNameString());
                    ret.append(" ");
                } else if ( (seg[0].getSectorListVal(1) ==
                                                seg[1].getSectorListVal(0)) ||
                            (seg[0].getSectorListVal(1) ==
                                                seg[1].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[0].getSectorListVal(1) +
                                        " is visible to this point" +
                                        " (seg[0].getSectorListVal(1))");
                    }
                    ret.append(s[seg[0].getSectorListVal(1)].getNameString());
                    ret.append(" ");
                } else {
                    the.asNeededPrintln("Unable to figure out what sector is " +
                                        "shared by these two segments");
                }
            } else if ((C0 <= DEG120) && (C2 <= DEG120)) {
                if ((seg[0].getSectorListVal(0) ==
                                                seg[2].getSectorListVal(0)) ||
                    (seg[0].getSectorListVal(0) ==
                                                seg[2].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[0].getSectorListVal(0) +
                                        " is visible to this point" +
                                        " (seg[0].getSectorListVal(0))");
                    }
                    ret.append(s[seg[0].getSectorListVal(0)].getNameString());
                    ret.append(" ");
                } else if ( (seg[0].getSectorListVal(1) ==
                                                seg[2].getSectorListVal(0)) ||
                            (seg[0].getSectorListVal(1) ==
                                                seg[2].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[0].getSectorListVal(1) +
                                        " is visible to this point" +
                                        " (seg[0].getSectorListVal(1))");
                    }
                    ret.append(s[seg[0].getSectorListVal(1)].getNameString());
                    ret.append(" ");
                } else {
                    the.asNeededPrintln("Unable to figure out what sector is " +
                                        "shared by these two segments");
                }
            } else if ((C1 <= DEG120) && (C2 <= DEG120)) {
                if ((seg[1].getSectorListVal(0) ==
                                                seg[2].getSectorListVal(0)) ||
                    (seg[1].getSectorListVal(0) ==
                                                seg[2].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[1].getSectorListVal(0) +
                                        " is visible to this point" +
                                        " (seg[1].getSectorListVal(0))");
                    }
                    ret.append(s[seg[1].getSectorListVal(0)].getNameString());
                    ret.append(" ");
                } else if ( (seg[1].getSectorListVal(1) ==
                                                seg[2].getSectorListVal(0)) ||
                            (seg[1].getSectorListVal(1) ==
                                                seg[2].getSectorListVal(1))) {
                    if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                        the.asNeededPrintln("Sector: " +
                                        seg[1].getSectorListVal(1) +
                                        " is visible to this point" +
                                        " (seg[1].getSectorListVal(1))");
                    }
                    ret.append(s[seg[1].getSectorListVal(1)].getNameString());
                    ret.append(" ");
                } else {
                    the.asNeededPrintln("Unable to figure out what sector is " +
                                        "shared by these two segments");
                }
            }
        }
        return ret.toString();
    }


    /**
     * This is the simplified determination of visible sectors.
     *
     * This method depends on the sector's direction and the angle
     * from the base station to the point (x,y).
     * It does not use the Law of Cosines.
     *
     * A sector's direction attribute indicates the center of the 120
     * degree range of the sector antenna.  This means the range visible
     * by the sector is from (direction - 60 degrees) to (direction + 60).
     *
     * Calculating the direction from base station to (x, y) can be done
     * by just calling the user getArriveDirection() method.  This was
     * a private class.  Since it may be used here, this method will be
     * moved out to the Util class and will be renamed getDirection().
     */
    private String calcVisibleSectorsOptimum(double x, double y, Sector [] s,int usr) {
        StringBuffer ret = new StringBuffer();;
        double userDir, segDir, loDir, hiDir;
        double bx, by;

        double DEG120 = 120 / the.RAD_TO_DEG;

//        System.out.println("120 degrees = " + DEG120);

        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            the.asNeededPrintln(
            "inside calcVisibleSectorsOptimum. x = " + x + " y = " + y);
        }

        for (int i = 0; i < s.length; i++) { // for each sector...
            if (s[i].getName() == -1) {
                break;
            }
			the.users[usr].SetVisibleSector(i,0); //Clear old 
            bx = s[i].getBaseX();
            by = s[i].getBaseY();

            userDir = Util.getDirection(bx, by, x, y, the.RAD_TO_DEG);
            segDir = s[i].getDirection();
			
			//Jonas
            loDir = segDir - (60+the.getSectorOverlap());
            hiDir = segDir + (60+the.getSectorOverlap());

            if (DebugLevel.debugging(DebugLevel.VISIBLE_DEBUGGING)) {
                the.asNeededPrintln(
                "segment: " + i +
                ", userDir: " + userDir + ", loDir: " + loDir +
                ", hiDir: " + hiDir);
            }

	    // need to handle the cases where a sector points to a negative
	    // direction.
		// if loDir < 0, then find the two pieces:
		// loDir1 = 360 - loDir, hiDir1 = 360
		// loDir2 = 0, hiDir2 stays at hiDir.
	    if (loDir < 0) {
		double loDir1 = 360 + loDir;  // loDir is negative!
		double hiDir1 = 360;
		double loDir2 = 0;
		double hiDir2 = hiDir;
            if (((loDir1 <= userDir) && (userDir < hiDir1)) ||
		    	((loDir2 <= userDir) && (userDir < hiDir2))) {
                    ret.append(s[i].getNameString());
                    ret.append(" ");
					the.users[usr].SetVisibleSector(i,1);
			}
		} else if(hiDir > 360){
			if(((loDir <= userDir) && (userDir <=360)) ||
				(userDir <= hiDir-360)){
					ret.append(s[i].getNameString());
                    ret.append(" ");
					the.users[usr].SetVisibleSector(i,1);
			}
	    } else {
                if ((loDir <= userDir) && (userDir < hiDir)) {
                    ret.append(s[i].getNameString());
                    ret.append(" ");
					the.users[usr].SetVisibleSector(i,1);
                }
	    }
        }
        return ret.toString();
    }

    /**
     * returns an array of segments which share with point bx and by
     * Generally used to find the edges touching the base XY location
     * <p>
     * @param ln the line segment array of all sectors
     * @param bx the x point
     * @param by the y point
     * @return the array of segments which share the point (bx,by)
     */
    private LineSegment [] findSegs(LineSegment [] ln, double bx, double by) {
        LineSegment [] ret = new LineSegment [3];
        int j = 0;
        if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
            the.asNeededPrintln(
            "inside findSegs. bx = " + bx + " by = " + by);
        }

        for (int i = 0; i < ln.length; i++) {
            if (Util.isEquiv(ln[i].getX1(), -1)) {
                break;
            }
            if (((Util.isEquiv(ln[i].getX1(), bx)) &&
                 (Util.isEquiv(ln[i].getY1(), by))) ||
                ((Util.isEquiv(ln[i].getX2(), bx)) &&
                 (Util.isEquiv(ln[i].getY2(), by)))) {
                if (DebugLevel.debugging(DebugLevel.FLOW_DEBUGGING)) {
                    the.asNeededPrintln(
            "inside findSegs for loop. i = " + i + " j = " + j);
                }

                ret[j] = new LineSegment(ln[i].getName(), ln[i].getX1(),
                            ln[i].getY1(), ln[i].getX2(), ln[i].getY2(),
                            ln[i].getSectorListVal(0));
                ret[j].setSectorListVal(1, ln[i].getSectorListVal(1));
                j++;
            }
        }

        return ret;
    }

    /**
     * This method determines what boundary cross information write out
     * to the appropriate boundary output file.
     * <p>
     * first, intesection is determined using this user's move start
     * and end point as one line segment and checking with the lines
     * of the line segement array to see which intersect with this user's
     * movement.
     * <p>
     * once found, the time of crossing is calculated using the
     * getTimeOfCross method.
     * <p>
     * NOTE:  If a user crosses two boundaries in a step, the antenna
     * of this user will be the one at the end of the step.  That is,
     * even though the user crosses through two sectors, the antenna
     * assigned is the current sector...
     * <p>
     * @param i this is the index into the user array for this user.
     */
    private void writeBoundaryResults(int i) {
        boolean foundIntersect = false;
        int prevSector, adjSector;
    // first need to find all Segments which are crossed by user movement
        LineSegment [] ln = the.model.getLineSeg();

        /*
         * In order to most quickly find the correct line intersection, use
         * the xLocation and yLocation circular array in User class.
         * Cur XY location will be at index cur.
         * the 10th previous will be at cur+1...
         */
        /*
        int cur = the.users[i].getCur();
        LineSegment userMove = new LineSegment(i,
                                the.users[i].getXLocation((cur+1)%10),
                                the.users[i].getYLocation((cur+1)%10),
                                the.users[i].getXLocation(cur),
                                the.users[i].getYLocation(cur),
                                -1);
        */
        LineSegment userMove = new LineSegment(i,
                                the.users[i].getPrevXLocation(),
                                the.users[i].getPrevYLocation(),
                                the.users[i].getCurXLocation(),
                                the.users[i].getCurYLocation(),
                                -1);

        for (int j = 0; j < ln.length; j++) {
            if (DebugLevel.debugging(DebugLevel.SEGMENT_DEBUGGING)) {
                the.asNeededPrintln("ln[" + j + "]: " + ln[j].toString());
            }
            if (ln[j].getX1() != -1) {
                if (userMove.intersects(ln[j], the.users[i])) {
                    foundIntersect = true;
                    // check to see what sector this cross leads to...
                    if (ln[j].getSectorListVal(0) ==
                                        the.users[i].getPrevSector()) {
                        writeBoundaryEntry(i, foundIntersect,
                                ln[j].getSectorListVal(1));
                    } else {
                        if (ln[j].getSectorListVal(1) ==
                                        the.users[i].getCurSector()) {
                            writeBoundaryEntry(i, foundIntersect,
                                the.users[i].getCurSector());
                        } else {
                            writeBoundaryEntry(i, foundIntersect,
                                ln[j].getSectorListVal(0));
                        }
                    }
                    if (DebugLevel.debugging(
                        DebugLevel.INTERSECT_DEBUGGING)) {
                        the.asNeededPrintln("user[" + i +
                            "] intersects line[" + j + "] at (" +
                            the.users[i].getXBoundaryCross() + "," +
                            the.users[i].getYBoundaryCross() + ")");
                    }
                }
            } else {
                // done because ln[j].getX1() is -1
                break;
            }
        }
        if (foundIntersect = false) {
            System.out.println("No intersection found!");
            writeBoundaryEntry(i, foundIntersect,
                the.users[i].getCurSector());
        }
    }
    /**
     * This method actually writes the boundary cross information out to the
     * appropriate boundary output file.
     * <p>
     * user boundary cross data is calculated by writeBoundaryResults()
     * which then calls this method.
     * <p>
     * @param i this is the index into the user array for this user.
     * @param foundIntersection should always be true... however if
     * false, then data may still be written..
     */
    private void writeBoundaryEntry(int i, boolean foundIntersect,
                                    int toSector) {
        StringBuffer line = new StringBuffer();
        int timeIncrement = 0;
        timeIncrement = the.TIME_INCREMENT;
        if (foundIntersect) {
            double time;
            if(!Util.isEquiv(the.users[i].getCurXLocation(),
		            the.users[i].getPrevXLocation())) {
                time = getTimeOfCross(    the.users[i].getCurXLocation(),
                                        the.users[i].getPrevXLocation(),
                                        the.users[i].getXBoundaryCross(),
                                        timeIncrement);
            } else { // vertical line so use Y locations.
                time = getTimeOfCross(    the.users[i].getCurYLocation(),
                                        the.users[i].getPrevYLocation(),
                                        the.users[i].getYBoundaryCross(),
                                        timeIncrement);
            }
            time = Math.abs(time);
            line.append(new Format("%4d").form(curStep));
            line.append(" ");
            line.append(new Format("%4d").form(i));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getXBoundaryCross()));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getYBoundaryCross()));
            line.append(" ");
            line.append(new Format("%3d").form(the.users[i].getCurMoveSpeed()));
            line.append(" ");
            line.append(new Format("%8.2f").form(the.users[i].getCurMoveDirection()));
            line.append(" ");
            line.append(new Format("%10.2f").form(
                ((curStep - 1) * the.TIME_INCREMENT) + time));
            line.append(" ");
            line.append(new Format("%4.2f").form(the.users[i].getCurPower()));
            line.append(" ");
            line.append(new Format("%c").form(the.users[i].getIsCallingDRC()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getPrevSectorString()));
            line.append(" ");
            line.append(new Format("%3s").form(Util.getSectorString(toSector)));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurAntennaString()));

            oboundary.println(line.toString());
        } else {
            the.asNeededPrintln("*** error finding intersection!" +
                    "will use previous location for (x,y) boundary cross");
            the.asNeededPrintln("*** user[" + i + "] Prev (" +
                the.users[i].getPrevXLocation() + "," +
                the.users[i].getPrevYLocation() + ") Cur (" +
                the.users[i].getCurXLocation() + "," +
                the.users[i].getCurYLocation() + ")");
            the.asNeededPrintln("*** user: " + the.users[i]);

/*
            if (DebugLevel.debugging(DebugLevel.INTERSECT_DEBUGGING)) {
                the.asNeededPrintln("*** error finding intersection!" +
                    "will use previous location for (x,y) boundary cross");
                the.asNeededPrintln("*** user[" + i + "] Prev (" +
                    the.users[i].getPrevXLocation() + "," +
                    the.users[i].getPrevYLocation() + ") Cur (" +
                    the.users[i].getCurXLocation() + "," +
                    the.users[i].getCurYLocation() + ")");
            }
*/
            line.append(new Format("%4d").form(curStep));
            line.append(" ");
            line.append(new Format("%4d").form(i));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getPrevXLocation()));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getPrevYLocation()));
            line.append(" ");
            line.append(new Format("%3d").form(the.users[i].getCurMoveSpeed()));
            line.append(" ");
            line.append(new Format("%8.2f").form(the.users[i].getCurMoveDirection()));
            line.append(" ");
            // currently user.setNextStats is hard coded for 10
            // second increments in the model

            // since this is the exception case, use previous step's time
            line.append(new Format("%10.2f").form(
                (float)((curStep - 1) * the.TIME_INCREMENT)));
            line.append(" ");
            line.append(new Format("%4.2f").form(the.users[i].getCurPower()));
            line.append(" ");
            line.append(new Format("%c").form(the.users[i].getIsCallingDRC()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getPrevSectorString()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurSectorString()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurAntennaString()));

            oboundary.println(line.toString());

        }

    }


    /**
     * This method writes the boundary cross information out to the
     * appropriate boundary output file.
     * <p>
     * This is the optimized version of this method.  This method
     * will not traverse and calculate every line segment in the array.
     * Instead, it will use the user's current sector location to find
     * and examine only line segments which are part of the current
     * sector.
     * <p>
     * It may have been possible to figure out exactly which of the
     * 4 line segments of the current sector to check based on the
     * user's movement direction.  However, this would require a total
     * of 3 calculations every time.  Instead, by trying to check all
     * 4 of the sector lines, the average will be only 2 calculations!
     * <p>
     * The intersects test is performed on each line segment and the
     * line segment of the user's movement until the intersected segment
     * is found.  Once found, see if the adjacent sector is the current
     * sector of this user.  If it is, then done, Otherwise, do this
     * same check on this new sector...
     * <p>
     * once found, the time of crossing is calculated using the
     * getTimeOfCross method.
     * <p>
     * NOTE:  If a user crosses two boundaries in a step, the antenna
     * of this user will be the one at the end of the step.  That is,
     * even though the user crosses through two sectors, the antenna
     * assigned is the current sector...
     * <p>
     * @param i this is the index into the user array for this user.
     */
    private void writeBoundaryResultsOptimum(int i) {
        boolean foundIntersect = false;
        int j;
        int prevSector;
        int curSector;
        int adjSector;

    // first need to find all Segments which are crossed by user movement
        LineSegment [] ln = the.model.getLineSeg();
        LineSegment userMove = new LineSegment(i,
                                the.users[i].getPrevXLocation(),
                                the.users[i].getPrevYLocation(),
                                the.users[i].getCurXLocation(),
                                the.users[i].getCurYLocation(),
                                -1);

        prevSector = the.users[i].getPrevSector();
        curSector = the.users[i].getCurSector();
        adjSector = -1;
        // start looking for adjacent sectors to prevSector.
      while (curSector != adjSector) {
        for (j = 0; (j < ln.length) && (ln[j].getX1() != -1); j++) {
                // if there are more ln's to get i.e. ln != -1

            if (DebugLevel.debugging(DebugLevel.SEGMENT_DEBUGGING)) {
                the.asNeededPrintln("ln[" + j + "]: " + ln[j].toString());
            }
            if (!crossedLines[j]) {
                // if this line hasn't already been crossed.

                // if prev is the 0th element
                if (prevSector == ln[j].getSectorListVal(0)) {

                    // check for intersection and only go on if there is
                    if (userMove.intersects(ln[j], the.users[i])) {
                        foundIntersect = true;
                        crossedLines[j] = true;
                        if (DebugLevel.debugging(
                            DebugLevel.INTERSECT_DEBUGGING)) {
                            the.asNeededPrint("user[" + i +
                                "] intersects line[" + j + "] at (" +
                                the.users[i].getXBoundaryCross() + "," +
                                the.users[i].getYBoundaryCross() + ")" +
                                "  j = " + j + " 1st half");
                                // the.asNeededPrintln("ln[" + j + "]: " +
                                //    ln[j].toString());

                        }

                        // check to see what sector this cross leads to...
                        if (ln[j].getSectorListVal(1) == curSector) {
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln(" 1st if");
                            }
                            adjSector = curSector;  // done
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln("    adjSector: " +
                                    adjSector + ", curSector: " +
                                    curSector + ", prevSector: " +
                                    prevSector);
                            }
                            writeBoundaryEntryOptimum(
                                i, foundIntersect, prevSector, curSector);
                        } else {
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln(" 2nd if");
                            }
                            adjSector = ln[j].getSectorListVal(1);
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln("    adjSector: " +
                                    adjSector + ", curSector: " +
                                    curSector + ", prevSector: " +
                                    prevSector);
                            }
                            writeBoundaryEntryOptimum(
                                i, foundIntersect, prevSector, adjSector);
                            prevSector = adjSector;
                        }
                        j = ln.length; // force for loop break
                    }
                } else if (prevSector == ln[j].getSectorListVal(1)) {
                    if (userMove.intersects(ln[j], the.users[i])) {
                        foundIntersect = true;
                        crossedLines[j] = true;

                        if (DebugLevel.debugging(
                            DebugLevel.INTERSECT_DEBUGGING)) {
                            the.asNeededPrint("user[" + i +
                                "] intersects line[" + j + "] at (" +
                                the.users[i].getXBoundaryCross() + "," +
                                the.users[i].getYBoundaryCross() + ")" +
                                "  j = " + j + " 2nd half");
                                // the.asNeededPrintln("ln[" + j + "]: " +
                                //    ln[j].toString());
                        }

                        // check to see what sector this cross leads to.
                        if (ln[j].getSectorListVal(0) == curSector) {
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln(" 1st if");
                            }
                            adjSector = curSector;  // done
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln("    adjSector: " +
                                    adjSector + ", curSector: " +
                                    curSector + ", prevSector: " +
                                    prevSector);
                            }
                            writeBoundaryEntryOptimum(
                                i, foundIntersect, prevSector, curSector);
                        } else {
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln(" 2nd if");
                            }
                            adjSector = ln[j].getSectorListVal(0);
                            if (DebugLevel.debugging(
                                DebugLevel.INTERSECT_DEBUGGING)) {
                                the.asNeededPrintln("    adjSector: " +
                                    adjSector + ", curSector: " +
                                    curSector + ", prevSector: " +
                                    prevSector);
                            }
                            writeBoundaryEntryOptimum(
                                i, foundIntersect, prevSector, adjSector);
                            prevSector = adjSector;
                        }
                        j = ln.length; // force for loop break
                    }
                }
            }
        }
      }
    }
    /**
     * This method actually writes the boundary cross information out to the
     * appropriate boundary output file.
     * <p>
     * user boundary cross data is calculated by writeBoundaryResultsOptimum()
     * which then calls this method.
     * <p>
     * @param i this is the index into the user array for this user.
     * @param foundIntersection should always be true... however if
     * false, then data may still be written..
     */
    private void writeBoundaryEntryOptimum(int i, boolean foundIntersect,
                                    int fromSector, int toSector) {
        StringBuffer line = new StringBuffer();
        int timeIncrement = 0;
        timeIncrement = the.TIME_INCREMENT;
        if (foundIntersect) {
            double time;
            if(!Util.isEquiv(the.users[i].getCurXLocation(),
		        the.users[i].getPrevXLocation())) {
                time = getTimeOfCross(    the.users[i].getCurXLocation(),
                                        the.users[i].getPrevXLocation(),
                                        the.users[i].getXBoundaryCross(),
                                        timeIncrement);
            } else { // vertical line so use Y locations.
                time = getTimeOfCross(    the.users[i].getCurYLocation(),
                                        the.users[i].getPrevYLocation(),
                                        the.users[i].getYBoundaryCross(),
                                        timeIncrement);
            }
            time = Math.abs(time);
            line.append(new Format("%4d").form(curStep));
            line.append(" ");
            line.append(new Format("%4d").form(i));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getXBoundaryCross()));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getYBoundaryCross()));
            line.append(" ");
            line.append(new Format("%3d").form(the.users[i].getCurMoveSpeed()));
            line.append(" ");
            line.append(new Format("%8.2f").form(the.users[i].getCurMoveDirection()));
            line.append(" ");
            line.append(new Format("%10.2f").form(
                ((curStep - 1) * the.TIME_INCREMENT) + time));
            line.append(" ");
            line.append(new Format("%4.2f").form(the.users[i].getCurPower()));
            line.append(" ");
            line.append(new Format("%c").form(the.users[i].getIsCallingDRC()));
            line.append(" ");
            line.append(new Format("%3s").form(Util.getSectorString(fromSector)));
            line.append(" ");
            line.append(new Format("%3s").form(Util.getSectorString(toSector)));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurAntennaString()));

            oboundary.println(line.toString());
        } else {
            the.asNeededPrintln("*** error finding intersection!" +
                    "will use previous location for (x,y) boundary cross");
            the.asNeededPrintln("*** user[" + i + "] Prev (" +
                the.users[i].getPrevXLocation() + "," +
                the.users[i].getPrevYLocation() + ") Cur (" +
                the.users[i].getCurXLocation() + "," +
                the.users[i].getCurYLocation() + ")");
            the.asNeededPrintln("*** user: " + the.users[i]);

/*
            if (DebugLevel.debugging(DebugLevel.INTERSECT_DEBUGGING)) {
                the.asNeededPrintln("*** error finding intersection!" +
                    "will use previous location for (x,y) boundary cross");
                the.asNeededPrintln("*** user[" + i + "] Prev (" +
                    the.users[i].getPrevXLocation() + "," +
                    the.users[i].getPrevYLocation() + ") Cur (" +
                    the.users[i].getCurXLocation() + "," +
                    the.users[i].getCurYLocation() + ")");
            }
*/
            line.append(new Format("%4d").form(curStep));
            line.append(" ");
            line.append(new Format("%4d").form(i));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getPrevXLocation()));
            line.append(" ");
            line.append(new Format("%6.2f").form(the.users[i].getPrevYLocation()));
            line.append(" ");
            line.append(new Format("%3d").form(the.users[i].getCurMoveSpeed()));
            line.append(" ");
            line.append(new Format("%8.2f").form(the.users[i].getCurMoveDirection()));
            line.append(" ");
            // currently user.setNextStats is hard coded for 10
            // second increments in the model

            // since this is the exception case, use previous step's time
            line.append(new Format("%10.2f").form(
                (float)((curStep - 1) * the.TIME_INCREMENT)));
            line.append(" ");
            line.append(new Format("%4.2f").form(the.users[i].getCurPower()));
            line.append(" ");
            line.append(new Format("%c").form(the.users[i].getIsCallingDRC()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getPrevSectorString()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurSectorString()));
            line.append(" ");
            line.append(new Format("%3s").form(the.users[i].getCurAntennaString()));

            oboundary.println(line.toString());

        }
    }

    /**
     * This method calculates the time of crossing based on the ratio
     * of the cross, xb, between the x1 and x2 locations and a time
     * increment of sec.
     * <p>
     * @param x1 first point of line (curX)
     * @param x2 second point of line (prevX)
     * @param xb point inbetween the two points above
     * @param sec the time increment in seconds
     */
    private double getTimeOfCross(double x1, double x2, double xb, int sec) {
        return (sec * ((xb - x2)/(x2 - x1)));
    }

    /**
     * this method clears all fields in the statistics area of this panel
     */
    public void clearAllFields() {
	westSide.setTitleLabel(
            "          Selected User Status           ");
        uSpeedText.setText("");
        uDirectionText.setText("");
        uCurLocText.setText("");
        uPrevLocText.setText("");
        uCallingText.setText("");
        uCurPowerText.setText("");
        uPrevPowerText.setText("");
        uCurAntennaText.setText("");
        uPrevAntennaText.setText("");

        uTotal.setText("0 steps so far");

    }

    /**
     * this method generates the date string used in creating the current
     * model's directory.
     */
    private String getDateString() {
        Date d = new Date();
        String date = new Format("%02d").form(d.getYear()) +
            new Format("%02d").form(d.getMonth() + 1) +
            new Format("%02d").form(d.getDate()) +
            new Format("%02d").form(d.getHours()) +
            new Format("%02d").form(d.getMinutes()) +
            new Format("%02d").form(d.getSeconds());
        return date;
    }
    /**
     * this method updates the values in the user statistics area of
     * this panel to be that of the user u.
     * <p>
     * @param u user to display.
     */
    public void updateFields(User u) {
	westSide.setTitleLabel("Selected User Status for User " + u.getName());
        setUSpeedText(new Double(u.getCurMoveSpeed()).toString());
        setUDirectionText(
            new Double(u.getCurMoveDirection()).toString());
        if (u.getIsCalling() == User.CALLING) {
            setUCallingText("Yes");
        } else { // ROAMING or DRIVING
            setUCallingText("No");
        }

        setUCurPowerText(new Double(u.getCurPower()).toString());
        setUPrevPowerText(new Double(u.getPrevPower()).toString());

        setUCurAntennaText(u.getCurAntennaString());
        setUPrevAntennaText(u.getPrevAntennaString());

        setUPrevLocText(u.getPrevSectorString());
        setUCurLocText(u.getCurSectorString());

		setCurHandoffText(u.getCurHandoffString());
		setPrevHandoffText(u.getPrevHandoffString());

//	the.POWER_MAX = 7.0;  // simply to test legend.
//	the.legend.updateLegend();
    }

    public void waitForPocatDone() {
        boolean there = false;

        while (!there) {
            if (new File(pocatDoneFilename.toString()).exists()) {
                there = true;
                System.out.println(pocatDoneFilename.toString() +
                                    " is there");
            } else {
                System.out.println(pocatDoneFilename.toString() +
                                    " is not there there - sleep");
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    System.err.println(e);
                    System.err.println("Unable to sleep successfully");
                }

                System.out.println("Woke up...");
            }
        }

    }

    public void waitForRacesimDone() {

    }

	private void writeModelDataFile() {
        StringBuffer line = new StringBuffer();

        try 
		{
			odataFile = new PrintStream(
            new FileOutputStream(modelDataFilename));
        } 
		catch (Exception e) 
		{
            the.asNeededPrintln("Problem when opening " + modelDataFilename);
        }
        odataFile.println(date);
		odataFile.println(curStep);
		odataFile.println(the.map.getNumUsers());
		odataFile.println(the.model.getPattern());
		odataFile.println(the.model.getBehavior());
		odataFile.println(the.model.getModel());
        //dataFile.println(the.RADIUS);
		odataFile.println(the.getSectorOverlap());

        odataFile.close();
    }

	public void readModelDataFile(String Filename){

		String[] value = new String [7];
		String line;
		
		the.setCursor(Frame.WAIT_CURSOR);

		String tmp = new String("utmost_files" + the.FILE_SEP +
              "activity" + the.FILE_SEP + Filename + the.FILE_SEP + "model_data");
		
		if(new File(tmp).exists()){
			openDataFile(tmp);
			try {
				int i=0;
				while ((line = idataFile.readLine()) != null) {
					value[i++] = line.trim();
				}
				idataFile.close();
			} catch (IOException e) {
				System.err.println(e);
				return;
			}
			date = new String(value[0]);
			
			//curStep = Integer.parseInt(value[1]);
			the.model.setNumUsers(value[2]);
			the.model.setPattern(value[3]);
			the.model.setBehavior(value[4]);
			the.model.setModel(value[5]);
			//dataFile.println(the.RADIUS);
			the.setSectorOverlap(Integer.parseInt(value[6]));
			//Update model menus
			the.model.updateToLoaded();
			curStep = Integer.parseInt(value[1]);
			curStepDisplayed = curStep;
		    firstGo = false;
		
			activityFilename.setLength(0);
		    doneFilename.setLength(0);
			activityFilename.append("utmost_files" + the.FILE_SEP +
                                        "activity" + the.FILE_SEP);
			activityFilename.append(date);
			modelDataFilename = new String(activityFilename.toString() +
                                                the.FILE_SEP + "model_data");
			activityFilename.append(the.FILE_SEP + curStepDisplayed);
			doneFilename.append(activityFilename + "done");
			pocatFilename.setLength(0);
			pocatDoneFilename.setLength(0);
			pocatFilename.append("other_sim_files" + the.FILE_SEP +
                                     "chan_assign" + the.FILE_SEP);
			pocatFilename.append(date);
			pocatFilename.append(the.FILE_SEP + curStepDisplayed);
			pocatDoneFilename.append(pocatFilename + "done");

		
			if (new File(pocatFilename.toString()).exists()){
				readPocatFile();
			}
			else
				System.out.println(pocatDoneFilename.toString() + " is not there");			
		
			readActivityFile();
			
			
            the.mapPanel.setDirText("Results in " + date);
			
			if(the.PowerControlType == 1) 
				PowerControl = new POCAT(the,the.PowerControlSetting,date);				
			
			the.map.update(the.map.getGraphics());
			uTotal.setText("Step "+ curStepDisplayed +" of " + curStep );
			the.setCursor(Frame.DEFAULT_CURSOR);
		}
    }

	private boolean openDataFile(String tmp){
		try {
			idataFile = new DataInputStream(new FileInputStream(tmp));
		} catch (Exception e) {
			the.asNeededPrintln("problem when opening " +tmp);
			return false;
		}
		return true;
	}


    public void readPocatFile() {
        String line;
        int step;
        int u;
		int prevNr;
		int curNr;
        double prevPwr;
        double curPwr;
        String prevAnt;
        String curAnt;
        
		openPocatStream();
        try {
            while ((line = iPocat.readLine()) != null) {
                line = line.trim();
                StringTokenizer tmp = new StringTokenizer(line);
                if (!tmp.hasMoreTokens()) {
                    break;
                }
                step = Integer.parseInt(tmp.nextToken());
                u = Integer.parseInt(tmp.nextToken());
                prevPwr = (Double.valueOf(tmp.nextToken())).doubleValue();
                curPwr = (Double.valueOf(tmp.nextToken())).doubleValue();
				prevNr = Integer.parseInt(tmp.nextToken());
                prevAnt = tmp.nextToken();
				for(int i=0;i<prevNr-1;i++)
				{
					the.users[u].setPrevHandoff(i,Util.getSectorInt(tmp.nextToken()));
				}
				curNr = Integer.parseInt(tmp.nextToken());
                curAnt = tmp.nextToken();
				for(int i=0;i<curNr-1;i++)
				{
					the.users[u].setCurHandoff(i,Util.getSectorInt(tmp.nextToken()));
				}


                System.out.println("step: " + step +
                                    " user: " + u +
                                    " prevPwr: " + prevPwr +
                                    " curPwr: " + curPwr +
                                    " prevAnt: " + prevAnt +
                                    " curAnt: " + curAnt);

                // update user with this info
                the.users[u].setPrevPower(prevPwr);
                the.users[u].setCurPower(curPwr);
                the.users[u].setPrevAntenna(Util.getSectorInt(prevAnt));
                the.users[u].setCurAntenna(Util.getSectorInt(curAnt));
				the.users[u].setPrevNumber(prevNr);
				the.users[u].setCurNumber(curNr);
            }
			iPocat.close();
        } catch (IOException e) {
            System.err.println(e);
            return;
        }
    }

	public void readActivityFile() {
        String line;
        int step;
        int u;
        double x;
        double y;

        openActivityStream();
        try 
		{
            while ((line = iActivity.readLine()) != null) 
			{
                line = line.trim();
                StringTokenizer tmp = new StringTokenizer(line);
                if (!tmp.hasMoreTokens()) 
				{
                    break;
                }
                step = Integer.parseInt(tmp.nextToken());
                u = Integer.parseInt(tmp.nextToken());
                x = (Double.valueOf(tmp.nextToken())).doubleValue();
                y = (Double.valueOf(tmp.nextToken())).doubleValue();
			

               System.out.println("step: " + step +
                                    " user: " + u +
                                    " x: " + x +
                                    " y: " + y );

               
                the.users[u].setCurXLocation(x);
                the.users[u].setCurYLocation(y);
				the.users[u].setCurMoveSpeed(Integer.parseInt(tmp.nextToken()));
				the.users[u].setCurMoveDirection((Double.valueOf(tmp.nextToken())).doubleValue());
				String str = new String(tmp.nextToken());//time
				str = new String(tmp.nextToken());//xmit
				str = new String(tmp.nextToken());//calling
				int calling = 0;
				switch (str.charAt(0)) {
					case 'D' : calling = 0; break;
					case 'R' : calling = 1; break;
					case 'C' : calling = 2; break;
					default : calling = 2;
				 }
				the.users[u].setIsCalling(calling);
				new String(tmp.nextToken());
				the.users[u].setCurSector(Util.getSectorInt(tmp.nextToken()));
				
				the.users[u].setCurVisibleSectors(calcVisibleSectorsOptimum(
                    the.users[u].getCurXLocation(), the.users[u].getCurYLocation(),
                    the.sector,u));
			
            }
			iActivity.close();
        } catch (IOException e) {
            System.err.println(e);
            return;
        }
    }

    public void readRacesimFile() {
        openRacesimStream();

    }

	public void setcurStep(int i){
		curStep=i;
	}

	private void setcurStepDisplayed(int i){
		curStepDisplayed=i;
	}

	public void startNew(){
		curStep = 0;
		curStepDisplayed = 0;
        firstGo = true;
	}
}
