/*
 * VariablesPanel class
 *
 * Author: M. Heidi McClure
 * Date Started: 3/12/97
 * Date Last Revised: 4/12/97
 */
import java.awt.*;
/**
 * VariablesPanel is a class which is planned to allow for custom settings
 * of various model variables.  That is, it would provide a way to set
 * parameters in the model like user dot size, cell radius meaning (currently
 * hardcoded at 5km radius), and time increment (currently hardcoded at
 * 10 seconds).
 * <p>
 * This class does not function completely.  It merely provides the
 * hooks to perform some parameterization of some of the variables used
 * in the model.
 * <p>
 * @author      M. Heidi McClure
 * @version UTMOST2 1.0
 * @see VariablesPanel
 * @see FilesPanel
 * @see SectorPanel
 * @see PropertySettings
 */
public class VariablesPanel extends Panel {

    private Label PanelTitle = new Label("Variables Used by UTMOST2");

    private Label vRadiusLabel =        new Label("Cell Radius (km):");
    private TextField vRadiusText =   new TextField("5");

    private Label vTimeIncLabel =       new Label("Time Increment (sec):");
    private TextField vTimeIncText =   new TextField("10");

    private Label vDotSizeLabel =       new Label("User Dot Size:");
    private TextField vDotSizeText =   new TextField("10");

    private Label vDirNameLabel = new Label("POCAT or RACESIM Directory:");
    private TextField vDirNameText =   new TextField("970314115541");

    private Label vPowerLabel = new Label("Max Power (in Watts) in Color Range");
    private CheckboxGroup powerCheckboxes = new CheckboxGroup();
    private Checkbox one = new Checkbox("1.0", powerCheckboxes, true);
    private Checkbox two = new Checkbox("2.0", powerCheckboxes, false);
    private Checkbox three = new Checkbox("3.0", powerCheckboxes, false);
    private Checkbox four = new Checkbox("4.0", powerCheckboxes, false);
    private Checkbox five = new Checkbox("5.0", powerCheckboxes, false);
    private Checkbox six = new Checkbox("6.0", powerCheckboxes, false);
    private Checkbox seven = new Checkbox("7.0", powerCheckboxes, false);

    private PropertySettings parent;
    /**
     * Constructor which will create a VariablesPanel object.
     *
     * @param p is the PropertySettings panel object which called this
     * constructor
     */
    public VariablesPanel(PropertySettings p) {
        parent = p;

        setLayout(null);

        resize(insets().left + insets().right + 462, insets().top + insets().bottom + 306);

        add(PanelTitle);
        PanelTitle.reshape(insets().left + 24,insets().top + 24,360,20);

        add(vRadiusLabel);
        vRadiusLabel.reshape(insets().left + 24,insets().top + 64,160,20);
        add(vRadiusText);
        vRadiusText.reshape(insets().left + 200,insets().top + 64,80,30);

        add(vTimeIncLabel);
        vTimeIncLabel.reshape(insets().left + 24,insets().top + 109,170,20);
        add(vTimeIncText);
        vTimeIncText.reshape(insets().left + 200,insets().top + 109,80,30);

        add(vDotSizeLabel);
        vDotSizeLabel.reshape(insets().left + 24,insets().top + 154,160,20);
        add(vDotSizeText);
        vDotSizeText.reshape(insets().left + 200,insets().top + 154,80,30);


		
        add(vDirNameLabel);
        vDirNameLabel.reshape(insets().left + 24,insets().top + 199,250,20);
        add(vDirNameText);
        vDirNameText.reshape(insets().left + 300,insets().top + 199,180,30);

        add(vPowerLabel);
        vPowerLabel.reshape(insets().left + 24,insets().top + 244,250,20);

	Panel checkPanel = new Panel();
	checkPanel.setLayout(new GridLayout(1, 7, 5, 20));
        checkPanel.add(one);
        checkPanel.add(two);
        checkPanel.add(three);
        checkPanel.add(four);
        checkPanel.add(five);
        checkPanel.add(six);
        checkPanel.add(seven);
	add(checkPanel);
        checkPanel.reshape(insets().left + 64,insets().top + 280,300,30);
    }

    public void setParent(PropertySettings p) {
        parent = p;
    }
    public void setUserDotSize(int i) {
        (parent.getUTMOST2()).DOT_SIZE = i;
    }
    public void setTimeInc(int i) {
        (parent.getUTMOST2()).TIME_INCREMENT = i;
    }
    public void setRadius(double d) {
        (parent.getUTMOST2()).RADIUS = d;
    }
    public void setDirName(String s) {
        (parent.getUTMOST2()).SIM_DIR_NAME = s;
    }
    public void setPowerMax(double p) {
        (parent.getUTMOST2()).POWER_MAX = p;
// System.out.println("power_max set to " + p);
    }

    /*
     * action() method is for handling Button, TextField, Choice
     * and CheckBox action events ONLY. Geary, pg. 72 and 82-.
     */
    public boolean action(Event evt, Object arg) {
            setRadius(new Integer(vRadiusText.getText()).intValue());
            setUserDotSize(new Integer(vDotSizeText.getText()).intValue());
            setTimeInc(new Integer(vTimeIncText.getText()).intValue());
            setDirName(vDirNameText.getText());
	    setPowerMax(new Double(
		(powerCheckboxes.getCurrent()).getLabel()).doubleValue());
/*
        if (evt.target == vRadiusText) {
            setRadius(new Integer(vRadiusText.getText()).intValue());
        }

        if (evt.target == vDotSizeText) {
            setUserDotSize(new Integer(vDotSizeText.getText()).intValue());
        }

        if (evt.target == vTimeIncText) {
            setTimeInc(new Integer(vTimeIncText.getText()).intValue());
        }

        if (evt.target == vDirNameText) {
            setDirName(vDirNameText.getText());
        }
*/
        return true;
    }

    public void updateValues() {
        setRadius(new Integer(vRadiusText.getText()).intValue());
        setUserDotSize(new Integer(vDotSizeText.getText()).intValue());
        setTimeInc(new Integer(vTimeIncText.getText()).intValue());
        setDirName(vDirNameText.getText());
	setPowerMax(new Double(
		(powerCheckboxes.getCurrent()).getLabel()).doubleValue());
	(parent.getUTMOST2()).legend.updateLegend();
	(parent.getUTMOST2()).map.updateMap();
    }

}

